package at.cl.atp.foresolution.converter.tptp;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import at.cl.atp.foresolution.model.Atom;
import at.cl.atp.foresolution.model.BinaryFormula;
import at.cl.atp.foresolution.model.Formula;
import at.cl.atp.foresolution.model.Functor;
import at.cl.atp.foresolution.model.NegatedFormula;
import at.cl.atp.foresolution.model.TermRegistry;
import at.cl.atp.tPTP.Annotated_formula;
import at.cl.atp.tPTP.CNF_annotated;
import at.cl.atp.tPTP.FOF_Formula;
import at.cl.atp.tPTP.TPTP_File;

public class TPTPConverter {

	// STATIC-----------------------------------------------------------
	public static Formula convert(final TPTP_File tptpFile) {
		return new TPTPConverter(tptpFile).doConvert();
	}

	// END STATIC ------------------------------------------------------

	private final TPTP_File input;
	private final TPTPtoFormulaSwitch tptpSwitch;
	private final Set<Formula> hypothesis;
	private final Set<Formula> conjectures;

	private TPTPConverter(final TPTP_File file) {
		this.input = file;
		this.tptpSwitch = new TPTPtoFormulaSwitch();
		this.hypothesis = new HashSet<Formula>();
		this.conjectures = new HashSet<Formula>();
	}

	private Formula doConvert() {
		Set<Formula> allFormulas = new HashSet<Formula>();
		for (Annotated_formula annotatedFormula : this.input.getInputs()) {
			if (annotatedFormula instanceof CNF_annotated) {
				CNF_annotated cnfAnnotated = (CNF_annotated) annotatedFormula;
				Formula formula = this.convertFormula(cnfAnnotated.getFormula());
				switch (cnfAnnotated.getRole()) {
				case CONJECTURE:
					this.conjectures.add(formula);
					break;
				case HYPOTHESIS:
					this.hypothesis.add(formula);
					break;
				case NEGATED_CONJECTURE:
					this.conjectures.add(formula);
					break;
				}
			} else {
				// TODO if other TPTP formula types are added, add them here too
			}
		}
		allFormulas.addAll(this.conjectures);
		allFormulas.addAll(this.hypothesis);
		Iterator<Formula> iterator = allFormulas.iterator();
		BinaryFormula root = TermRegistry.createConjunction(iterator.next(),
				this.constructMultivaluedConjunction(iterator));
		return root;
	}

	private Formula constructMultivaluedConjunction(final Iterator<Formula> iterator) {
		Formula left = iterator.next();
		Formula right = null;
		if (iterator.hasNext()) {
			right = this.constructMultivaluedConjunctionRec(iterator);
		}
		BinaryFormula conjunction = TermRegistry.createConjunction(left, right);
		return conjunction;
	}

	private Formula constructMultivaluedConjunctionRec(final Iterator<Formula> iterator) {
		Formula nextConjunct = iterator.next();
		if (iterator.hasNext()) {
			Formula left = nextConjunct;
			Formula right = this.constructMultivaluedConjunctionRec(iterator);
			BinaryFormula conjunction = TermRegistry.createConjunction(left, right);
			return conjunction;
		} else {
			return nextConjunct;
		}
	}

	private Formula convertFormula(final FOF_Formula fofFormula) {
		Formula result = this.tptpSwitch.doSwitch(fofFormula);
		return result;
	}

	@SuppressWarnings("unused")
	private Formula createTautology() {
		Atom x = TermRegistry.createVariable("x");
		Functor predicate = TermRegistry.createPredicate("P", x);
		NegatedFormula negation = TermRegistry.createNegatedFormula(predicate);
		BinaryFormula disjunction = TermRegistry.createDisjunction(predicate, negation);
		return disjunction;
	}
}
