package at.cl.atp.foresolution.converter.tptp;

import org.eclipse.emf.ecore.EObject;

import at.cl.atp.foresolution.model.BinaryFormula;
import at.cl.atp.foresolution.model.Formula;
import at.cl.atp.foresolution.model.NegatedFormula;
import at.cl.atp.foresolution.model.QuantifiedFormula;
import at.cl.atp.foresolution.model.TermRegistry;
import at.cl.atp.tPTP.Annotated_formula;
import at.cl.atp.tPTP.CNF_annotated;
import at.cl.atp.tPTP.FOF_Atom;
import at.cl.atp.tPTP.FOF_Constant;
import at.cl.atp.tPTP.FOF_Formula;
import at.cl.atp.tPTP.FOF_Function;
import at.cl.atp.tPTP.FOF_NegatedFormula;
import at.cl.atp.tPTP.FOF_Predicate;
import at.cl.atp.tPTP.FOF_QuantifiedFormula;
import at.cl.atp.tPTP.FOF_Variable;
import at.cl.atp.tPTP.TPTP_File;
import at.cl.atp.tPTP.util.TPTPSwitch;

public class TPTPtoFormulaSwitch extends TPTPSwitch<Formula> {

	@Override
	public Formula caseTPTP_File(final TPTP_File file) {
		return null;
	}

	@Override
	public Formula caseAnnotated_formula(final Annotated_formula object) {
		return null;
	}

	@Override
	public Formula caseCNF_annotated(final CNF_annotated object) {
		return null;
	}

	@Override
	public Formula caseFOF_Formula(final FOF_Formula object) {
		if (object.getOperation() == at.cl.atp.tPTP.BinaryOperation.IMPLICATION) {
			// perform implication elimination
			FOF_Formula fof_left = object.getLeft();
			FOF_Formula fof_right = object.getRight();
			Formula left = this.doSwitch(fof_left);
			Formula right = this.doSwitch(fof_right);
			NegatedFormula notLeft = TermRegistry.createNegatedFormula(left);
			BinaryFormula disjunction = TermRegistry.createDisjunction(notLeft, right);
			return disjunction;
		} else if (object.getOperation() != null) {
			FOF_Formula fof_left = object.getLeft();
			FOF_Formula fof_right = object.getRight();
			Formula left = this.doSwitch(fof_left);
			Formula right = this.doSwitch(fof_right);
			BinaryFormula formula = null;
			switch (object.getOperation()) {
			case CONJUNCTION:
				formula = TermRegistry.createConjunction(left, right);
				break;
			case DISJUNCTION:
				formula = TermRegistry.createDisjunction(left, right);
				break;
			default:
				throw new RuntimeException("This should be unreachable!");
			}
			return formula;
		} else {
			return this.doSwitch(object.getLeft());
		}
	}

	@Override
	public Formula caseFOF_QuantifiedFormula(final FOF_QuantifiedFormula object) {
		FOF_Formula fof_inner = object.getInnerFormula();
		Formula innerFormula = this.doSwitch(fof_inner);
		QuantifiedFormula result = null;
		switch (object.getQuantifier()) {
		case EXISTENTIAL:
			result = TermRegistry.createExistentialFormula(innerFormula, object.getVariables());
			break;
		case UNIVERSAL:
			result = TermRegistry.createExistentialFormula(innerFormula, object.getVariables());
			break;
		}
		return result;
	}

	@Override
	public Formula caseFOF_NegatedFormula(final FOF_NegatedFormula object) {
		FOF_Formula fof_inner = object.getInnerFormula();
		return TermRegistry.createNegatedFormula(this.doSwitch(fof_inner));
	}

	@Override
	public Formula caseFOF_Predicate(final FOF_Predicate object) {
		Formula[] newArgs = new Formula[object.getArguments().size()];
		for (int i = 0; i < newArgs.length; i++) {
			FOF_Formula oldArg = object.getArguments().get(i);
			newArgs[i] = this.doSwitch(oldArg);
		}
		return TermRegistry.createPredicate(object.getName(), newArgs);
	}

	@Override
	public Formula caseFOF_Function(final FOF_Function object) {
		Formula[] newArgs = new Formula[object.getArguments().size()];
		for (int i = 0; i < newArgs.length; i++) {
			FOF_Formula oldArg = object.getArguments().get(i);
			newArgs[i] = this.doSwitch(oldArg);
		}
		return TermRegistry.createFunction(object.getName(), newArgs);
	}

	@Override
	public Formula caseFOF_Atom(final FOF_Atom object) {
		return null;
	}

	@Override
	public Formula caseFOF_Variable(final FOF_Variable object) {
		return TermRegistry.createVariable(object.getName());
	}

	@Override
	public Formula caseFOF_Constant(final FOF_Constant object) {
		return TermRegistry.createConstant(object.getName());
	}

	@Override
	public Formula defaultCase(final EObject object) {
		throw new RuntimeException("Unknown class: " + object.getClass().getName());
	}
}
