package at.cl.atp.foresolution.converter.xtext;

import org.eclipse.emf.ecore.EObject;

import at.cl.atp.fOResolution.Atom;
import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.Constant;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.fOResolution.Variable;
import at.cl.atp.fOResolution.util.FOResolutionSwitch;
import at.cl.atp.foresolution.model.BinaryFormula;
import at.cl.atp.foresolution.model.Formula;
import at.cl.atp.foresolution.model.TermRegistry;

public class XtextToFormulaSwitch extends FOResolutionSwitch<Formula> {

	@Override
	public Formula caseFormula(final at.cl.atp.fOResolution.Formula object) {
		throw new RuntimeException("Case not implemented: " + object.getClass().getName());
	}

	@Override
	public Formula caseQuantifiedFormula(final QuantifiedFormula object) {
		at.cl.atp.fOResolution.Formula fof_inner = object.getInnerFormula();
		Formula innerFormula = this.doSwitch(fof_inner);
		at.cl.atp.foresolution.model.QuantifiedFormula result = null;
		switch (object.getQuantifier()) {
		case EXISTENTIAL:
			result = TermRegistry.createExistentialFormula(innerFormula, object.getVariables());
			break;
		case UNIVERSAL:
			result = TermRegistry.createUniversalFormula(innerFormula, object.getVariables());
			break;
		}
		return result;
	}

	@Override
	public Formula caseNegatedFormula(final NegatedFormula object) {
		at.cl.atp.fOResolution.Formula fof_inner = object.getInnerFormula();
		return TermRegistry.createNegatedFormula(this.doSwitch(fof_inner));
	}

	@Override
	public Formula casePredicate(final Predicate object) {
		Formula[] newArgs = new Formula[object.getArguments().size()];
		for (int i = 0; i < newArgs.length; i++) {
			at.cl.atp.fOResolution.Formula oldArg = object.getArguments().get(i);
			newArgs[i] = this.doSwitch(oldArg);
		}
		return TermRegistry.createPredicate(object.getName(), newArgs);
	}

	@Override
	public Formula caseFunction(final Function object) {
		Formula[] newArgs = new Formula[object.getArguments().size()];
		for (int i = 0; i < newArgs.length; i++) {
			at.cl.atp.fOResolution.Formula oldArg = object.getArguments().get(i);
			newArgs[i] = this.doSwitch(oldArg);
		}
		return TermRegistry.createFunction(object.getName(), newArgs);
	}

	@Override
	public Formula caseAtom(final Atom object) {
		throw new RuntimeException("Case not implemented: " + object.getClass().getName());
	}

	@Override
	public Formula caseVariable(final Variable object) {
		return TermRegistry.createVariable(object.getName());
	}

	@Override
	public Formula caseConstant(final Constant object) {
		return TermRegistry.createConstant(object.getName());
	}

	@Override
	public Formula caseLowBindFormula(final LowBindFormula object) {
		if (object.getOperation() == BinaryOperation.IMPLICATION) {
			// perform implication elimination
			at.cl.atp.fOResolution.Formula fof_left = object.getLeft();
			at.cl.atp.fOResolution.Formula fof_right = object.getRight();
			Formula left = this.doSwitch(fof_left);
			Formula right = this.doSwitch(fof_right);
			at.cl.atp.foresolution.model.NegatedFormula notLeft = TermRegistry.createNegatedFormula(left);
			BinaryFormula disjunction = TermRegistry.createDisjunction(notLeft, right);
			return disjunction;
		} else if (object.getOperation() != null) {
			at.cl.atp.fOResolution.Formula fof_left = object.getLeft();
			at.cl.atp.fOResolution.Formula fof_right = object.getRight();
			Formula left = this.doSwitch(fof_left);
			Formula right = this.doSwitch(fof_right);
			BinaryFormula formula = null;
			switch (object.getOperation()) {
			case CONJUNCTION:
				formula = TermRegistry.createConjunction(left, right);
				break;
			case DISJUNCTION:
				formula = TermRegistry.createDisjunction(left, right);
				break;
			default:
				throw new RuntimeException("This should be unreachable!");
			}
			return formula;
		} else {
			return this.doSwitch(object.getLeft());
		}
	}

	@Override
	public Formula defaultCase(final EObject object) {
		throw new RuntimeException("Unknown class: " + object.getClass().getName());
	}
}
