package at.cl.atp.foresolution.exceptions;

import at.cl.atp.foresolution.model.Formula;
import at.cl.atp.foresolution.model.NegatedFormula;
import at.cl.atp.foresolution.model.clause.Clause;
import at.cl.atp.foresolution.model.clause.ClauseLiteral;
import at.cl.atp.foresolution.model.substitution.Substitution;
import at.cl.atp.foresolution.model.unification.Unification;

public class ClashProducedBoxException extends RuntimeException {

	private static final long serialVersionUID = 1L;

	private Clause first;
	private Clause second;
	private ClauseLiteral firstLiteral;
	private ClauseLiteral secondLiteral;
	private Substitution mgu;

	public ClashProducedBoxException() {
		this.first = null;
		this.second = null;
		this.firstLiteral = null;
		this.secondLiteral = null;
	}

	public ClashProducedBoxException(final Clause first, final Clause second) {
		super();
		this.first = first;
		this.second = second;
		if (first.getLiterals().isEmpty() == false) {
			this.firstLiteral = first.getLiterals().iterator().next();
		} else {
			this.firstLiteral = first.getNegatedLiterals().iterator().next();
		}
		if (second.getLiterals().isEmpty() == false) {
			this.secondLiteral = second.getLiterals().iterator().next();
		} else {
			this.secondLiteral = second.getNegatedLiterals().iterator().next();
		}
		Formula firstFormula = null;
		Formula secondFormula = null;
		if (this.firstLiteral.isNegated()) {
			firstFormula = ((NegatedFormula) this.firstLiteral.getFormula()).getInnerFormula();
		} else {
			firstFormula = this.firstLiteral.getFormula();
		}
		if (this.secondLiteral.isNegated()) {
			secondFormula = ((NegatedFormula) this.secondLiteral.getFormula()).getInnerFormula();
		} else {
			secondFormula = this.secondLiteral.getFormula();
		}
		this.mgu = Unification.createMGU(firstFormula, secondFormula);
	}

	@Override
	public String getMessage() {
		if (this.first == null) {
			return "";
		}
		return "Reached BOX with: " + this.first.toString() + " and " + this.second.toString() + " on MGU "
				+ this.mgu.toString() + ".";
	}

	public Clause getFirst() {
		return this.first;
	}

	public void setFirst(final Clause first) {
		this.first = first;
	}

	public Clause getSecond() {
		return this.second;
	}

	public void setSecond(final Clause second) {
		this.second = second;
	}

	public ClauseLiteral getFirstLiteral() {
		return this.firstLiteral;
	}

	public void setFirstLiteral(final ClauseLiteral firstLiteral) {
		this.firstLiteral = firstLiteral;
	}

	public ClauseLiteral getSecondLiteral() {
		return this.secondLiteral;
	}

	public void setSecondLiteral(final ClauseLiteral secondLiteral) {
		this.secondLiteral = secondLiteral;
	}

	public Substitution getMgu() {
		return this.mgu;
	}

	public void setMgu(final Substitution mgu) {
		this.mgu = mgu;
	}

}
