package at.cl.atp.foresolution.model;

import at.cl.atp.foresolution.model.enums.BinaryOperation;

public class BinaryFormula extends Formula {

	private final BinaryOperation operation;
	private final Formula left;
	private final Formula right;

	/* default */BinaryFormula(final BinaryOperation operation, final long leftID, final long rightID) {
		this(operation, TermRegistry.getByID(leftID), TermRegistry.getByID(rightID));
	}

	/* default */BinaryFormula(final BinaryOperation operation, final Formula left, final Formula right) {
		this.operation = operation;
		this.left = left;
		this.right = right;
	}

	public BinaryOperation getOperation() {
		return this.operation;
	}

	public Formula getLeft() {
		return this.left;
	}

	public Formula getRight() {
		return this.right;
	}

	public long getLeftID() {
		return this.left.getFormulaID();
	}

	public long getRightID() {
		return this.right.getFormulaID();
	}

	@Override
	public boolean contentEquals(final Formula other) {
		// guarded by TermRegistry, we create each binary formula only once
		return other == this;
	}

	@Override
	public String toString() {
		return this.operation.toString() + "(" + this.left.toString() + ", " + this.right.toString() + ")";
	}
}
