package at.cl.atp.foresolution.model;

import at.cl.atp.foresolution.model.enums.AtomType;
import at.cl.atp.foresolution.model.enums.BinaryOperation;
import at.cl.atp.foresolution.model.enums.FunctorType;
import at.cl.atp.foresolution.model.enums.Quantifier;

/*default*/class FormulaFactory {

	public static Atom createVariable(final String name) {
		return new Atom(AtomType.VARIABLE, name);
	}

	public static Atom createConstant(final String name) {
		return new Atom(AtomType.CONSTANT, name);
	}

	public static BinaryFormula createConjunction(final Formula left, final Formula right) {
		return new BinaryFormula(BinaryOperation.AND, left, right);
	}

	public static BinaryFormula createDisjunction(final Formula left, final Formula right) {
		return new BinaryFormula(BinaryOperation.OR, left, right);
	}

	public static BinaryFormula createConjunction(final long leftID, final long rightID) {
		return new BinaryFormula(BinaryOperation.AND, leftID, rightID);
	}

	public static BinaryFormula createDisjunction(final long leftID, final long rightID) {
		return new BinaryFormula(BinaryOperation.OR, leftID, rightID);
	}

	public static Functor createFunction(final String name, final Formula... arguments) {
		return new Functor(FunctorType.FUNCTION, name, arguments);
	}

	public static Functor createPredicate(final String name, final Formula... arguments) {
		return new Functor(FunctorType.PREDICATE, name, arguments);
	}

	public static Functor createFunction(final String name, final long... argumentIDs) {
		return new Functor(FunctorType.FUNCTION, name, argumentIDs);
	}

	public static Functor createPredicate(final String name, final long... argumentIDs) {
		return new Functor(FunctorType.PREDICATE, name, argumentIDs);
	}

	public static QuantifiedFormula createUniversalFormula(final Formula innerFormula, final String quantifiedVariable) {
		return new QuantifiedFormula(Quantifier.UNIVERSAL, innerFormula, quantifiedVariable);
	}

	public static QuantifiedFormula createExistentialFormula(final Formula innerFormula, final String quantifiedVariable) {
		return new QuantifiedFormula(Quantifier.EXISTENTIAL, innerFormula, quantifiedVariable);
	}

	public static QuantifiedFormula createUniversalFormula(final long innerFormulaID, final String quantifiedVariable) {
		return new QuantifiedFormula(Quantifier.UNIVERSAL, innerFormulaID, quantifiedVariable);
	}

	public static QuantifiedFormula createExistentialFormula(final long innerFormulaID, final String quantifiedVariable) {
		return new QuantifiedFormula(Quantifier.EXISTENTIAL, innerFormulaID, quantifiedVariable);
	}

	public static NegatedFormula createNegatedFormula(final Formula innerFormula) {
		return new NegatedFormula(innerFormula);
	}

	public static NegatedFormula createNegatedFormula(final long innerFormulaID) {
		return new NegatedFormula(innerFormulaID);
	}
}
