package at.cl.atp.foresolution.model;

import at.cl.atp.foresolution.model.enums.FunctorType;
import at.cl.atp.foresolution.model.utils.FormulaFormatter;

public class Functor extends Formula {

	private final FunctorType type;
	private final String name;
	private final Formula[] arguments;

	/* default */Functor(final FunctorType type, final String name, final long... arguments) {
		this.type = type;
		this.name = name;
		this.arguments = new Formula[arguments.length];
		for (int i = 0; i < arguments.length; i++) {
			this.arguments[i] = TermRegistry.getByID(arguments[i]);
		}
	}

	/* default */Functor(final FunctorType type, final String name, final Formula... arguments) {
		this.type = type;
		this.name = name;
		this.arguments = arguments;
	}

	public FunctorType getType() {
		return this.type;
	}

	public Formula[] getArguments() {
		return this.arguments;
	}

	public long getArgumentID(final int argumentIndex) {
		return this.arguments[argumentIndex].getFormulaID();
	}

	public String getName() {
		return this.name;
	}

	@Override
	public boolean contentEquals(final Formula other) {
		// guarded by term registry: we create each functor only once
		return this == other;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append(FormulaFormatter.formatIdentifier(this.name));
		builder.append("(");
		for (int i = 0; i < this.arguments.length; i++) {
			builder.append(this.arguments[i].toString());
			if ((i + 1) < this.arguments.length) {
				builder.append(", ");
			}
		}
		builder.append(")");
		return builder.toString();
	}
}
