package at.cl.atp.foresolution.model;

import at.cl.atp.foresolution.model.enums.Quantifier;
import at.cl.atp.foresolution.model.utils.FormulaFormatter;

public class QuantifiedFormula extends Formula {

	private final Quantifier quantifier;
	private final Formula innerFormula;
	private final String quantifiedVariable;

	/* default */QuantifiedFormula(final Quantifier quantifier, final long innerFormulaID,
			final String quantifiedVariable) {
		this.quantifier = quantifier;
		this.innerFormula = TermRegistry.getByID(innerFormulaID);
		this.quantifiedVariable = quantifiedVariable;
	}

	/* default */QuantifiedFormula(final Quantifier quantifier, final Formula innerFormula,
			final String quantifiedVariable) {
		this.quantifier = quantifier;
		this.innerFormula = innerFormula;
		this.quantifiedVariable = quantifiedVariable;
	}

	public Quantifier getQuantifier() {
		return this.quantifier;
	}

	public Formula getInnerFormula() {
		return this.innerFormula;
	}

	public String getQuantifiedVariable() {
		return this.quantifiedVariable;
	}

	@Override
	public boolean contentEquals(final Formula other) {
		// guarded by TermRegistry, we create each quantified formula only once
		return other == this;
	}

	@Override
	public String toString() {
		return this.quantifier.toString() + "[" + FormulaFormatter.formatIdentifier(this.quantifiedVariable) + "]("
				+ this.innerFormula.toString() + ")";
	}
}
