package at.cl.atp.foresolution.model.clause;

import java.util.HashSet;
import java.util.Set;

import at.cl.atp.foresolution.model.BinaryFormula;
import at.cl.atp.foresolution.model.Formula;
import at.cl.atp.foresolution.model.container.FormulaContainer;
import at.cl.atp.foresolution.model.enums.BinaryOperation;
import at.cl.atp.foresolution.transformations.NormalFormTransformations;
import at.cl.atp.foresolution.transformations.TransformationType;

public class ClausalFormBuilder {

	public static ClausalForm build(final FormulaContainer input) {
		if (input.getAppliedTransformations().contains(TransformationType.CLAUSAL_FORM) == false) {
			NormalFormTransformations.toClausalForm(input);
		}
		ClausalForm clausalForm = new ClausalForm();
		buildClausesRec(input.getFormula(), clausalForm);
		return clausalForm;
	}

	private static void buildClausesRec(final Formula formula, final ClausalForm clausalForm) {
		if (formula instanceof BinaryFormula) {
			BinaryFormula f = (BinaryFormula) formula;
			if (f.getOperation() == BinaryOperation.AND) {
				buildClausesRec(f.getLeft(), clausalForm);
				buildClausesRec(f.getRight(), clausalForm);
				return;
			}
		}
		clausalForm.add(buildClause(formula));
	}

	private static Clause buildClause(final Formula formula) {
		Set<ClauseLiteral> literals = new HashSet<ClauseLiteral>();
		buildClauseRec(formula, literals);
		Clause clause = new Clause(null, null, null, literals);
		return clause;
	}

	private static void buildClauseRec(final Formula formula, final Set<ClauseLiteral> literals) {
		if (formula instanceof BinaryFormula) {
			BinaryFormula f = (BinaryFormula) formula;
			if (f.getOperation() == BinaryOperation.OR) {
				buildClauseRec(f.getLeft(), literals);
				buildClauseRec(f.getRight(), literals);
				return;
			}
		}
		ClauseLiteral literal = new ClauseLiteral(formula);
		literals.add(literal);
	}
}
