package at.cl.atp.foresolution.model.utils;

import java.util.HashMap;
import java.util.Map;

public class MultikeyMap<K, V> {

	private final int keySequenceLength;
	private final Map<K, Object> contents;

	public MultikeyMap(final int keySequenceLength) {
		this.keySequenceLength = keySequenceLength;
		this.contents = new HashMap<K, Object>();
	}

	@SuppressWarnings("unchecked")
	public void put(final V value, final K... keys) {
		if (keys.length != this.keySequenceLength) {
			throw new RuntimeException("Map needs key sequence length of " + this.keySequenceLength
					+ ", but these keys have length " + keys.length + "!");
		}
		Map<K, Object> map = this.contents;
		for (int i = 0; i < this.keySequenceLength; i++) {
			K key = keys[i];
			if ((i + 1) == this.keySequenceLength) {
				// set the value
				map.put(key, value);
			} else {
				// dig deeper
				Map<K, Object> innerMap = (Map<K, Object>) map.get(key);
				if (innerMap == null) {
					innerMap = new HashMap<K, Object>();
					map.put(key, innerMap);
				}
				map = innerMap;
			}
		}
	}

	@SuppressWarnings("unchecked")
	public V get(final K... keys) {
		if (keys.length != this.keySequenceLength) {
			throw new RuntimeException("Map needs key sequence length of " + this.keySequenceLength
					+ ", but these keys have length " + keys.length + "!");
		}
		Map<K, Object> map = this.contents;
		for (int i = 0; i < this.keySequenceLength; i++) {
			K key = keys[i];
			if ((i + 1) == this.keySequenceLength) {
				// get the value
				return (V) map.get(key);
			} else {
				// dig deeper
				Map<K, Object> innerMap = (Map<K, Object>) map.get(key);
				if (innerMap == null) {
					return null;
				}
				map = innerMap;
			}
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public boolean containsKey(final K... keys) {
		if (keys.length != this.keySequenceLength) {
			throw new RuntimeException("Map needs key sequence length of " + this.keySequenceLength
					+ ", but these keys have length " + keys.length + "!");
		}
		Map<K, Object> map = this.contents;
		for (int i = 0; i < this.keySequenceLength; i++) {
			K key = keys[i];
			if ((i + 1) == this.keySequenceLength) {
				// get the value
				return map.containsKey(key);
			} else {
				// dig deeper
				Map<K, Object> innerMap = (Map<K, Object>) map.get(key);
				if (innerMap == null) {
					return false;
				}
				map = innerMap;
			}
		}
		return false;
	}
}
