package at.cl.atp.foresolution.model.utils;

import java.util.HashMap;
import java.util.Map;

public class TernaryKeyMap<K1, K2, K3, V> {

	private final Map<K1, Map<K2, Map<K3, V>>> contents;

	public TernaryKeyMap() {
		this.contents = new HashMap<K1, Map<K2, Map<K3, V>>>();
	}

	public void put(final K1 key1, final K2 key2, final K3 key3, final V value) {
		Map<K2, Map<K3, V>> innerMap = this.contents.get(key1);
		if (innerMap == null) {
			innerMap = new HashMap<K2, Map<K3, V>>();
			this.contents.put(key1, innerMap);
		}
		Map<K3, V> innermostMap = innerMap.get(key2);
		if (innermostMap == null) {
			innermostMap = new HashMap<K3, V>();
			innerMap.put(key2, innermostMap);
		}
		innermostMap.put(key3, value);
	}

	public V get(final K1 key1, final K2 key2, final K3 key3) {
		Map<K2, Map<K3, V>> innerMap = this.contents.get(key1);
		if (innerMap == null) {
			return null;
		}
		Map<K3, V> innermostMap = innerMap.get(key2);
		if (innermostMap == null) {
			return null;
		}
		return innermostMap.get(key3);
	}

	public boolean containsKey(final K1 key1, final K2 key2, final K3 key3) {
		Map<K2, Map<K3, V>> innerMap = this.contents.get(key1);
		if (innerMap == null) {
			return false;
		}
		Map<K3, V> innermostMap = innerMap.get(key2);
		if (innermostMap == null) {
			return false;
		}
		return innermostMap.containsKey(key3);
	}
}
