package at.cl.atp.foresolution.parser;

import java.io.File;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import at.cl.atp.FOResolutionStandaloneSetup;
import at.cl.atp.TPTPStandaloneSetup;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.foresolution.converter.tptp.TPTPConverter;
import at.cl.atp.foresolution.converter.xtext.XtextConverter;
import at.cl.atp.tPTP.TPTP_File;

public class FirstOrderDSLParser {

	static {
		FOResolutionStandaloneSetup.doSetup();
		TPTPStandaloneSetup.doSetup();
	}

	private static Object parse(final File source) {
		ResourceSet resourceSet = new ResourceSetImpl();
		Object returnValue = null;
		URI fileURI = URI.createFileURI(source.getAbsolutePath());
		Resource resource = resourceSet.getResource(fileURI, true);
		returnValue = resource.getContents().get(0);
		return returnValue;
	}

	private static Formula parseFormula(final File source) {
		if (!isFormulaFile(source)) {
			throw new RuntimeException("The file " + source.getName() + " is not a Formula file!");
		}
		return (Formula) parse(source);
	}

	private static TPTP_File parseTPTP(final File source) {
		if (!isTPTPFile(source)) {
			throw new RuntimeException("The file " + source.getName() + " is not a TPTP file!");
		}
		return (TPTP_File) parse(source);
	}

	public static at.cl.atp.foresolution.model.Formula parseAndConvertToFormula(final File source) {
		if (isFormulaFile(source)) {
			return XtextConverter.convert(parseFormula(source));
		} else {
			TPTP_File tptpFile = parseTPTP(source);
			return TPTPConverter.convert(tptpFile);
		}
	}

	public static boolean isFormulaFile(final File source) {
		if (source.getName().endsWith(".fores")) {
			return true;
		} else {
			return false;
		}
	}

	public static boolean isTPTPFile(final File source) {
		if (source.getName().endsWith(".tptp")) {
			return true;
		} else {
			return false;
		}
	}
}
