package at.cl.atp.validation

import at.cl.atp.fOResolution.Constant
import at.cl.atp.fOResolution.Variable
import at.cl.atp.fOResolution.Function
import at.cl.atp.fOResolution.Predicate
import at.cl.atp.fOResolution.LowBindFormula
import at.cl.atp.fOResolution.QuantifiedFormula
import at.cl.atp.fOResolution.Quantifier
import at.cl.atp.fOResolution.NegatedFormula

class FormulaPrettyPrinter {
	
	public static def dispatch void print(Constant formula){
		System::out.print(formula.name);
	}
	
	public static def dispatch void print(Variable formula){
		System::out.print(formula.name);
	}
	
	public static def dispatch void print(QuantifiedFormula formula){
		switch(formula.quantifier){
			case Quantifier::EXISTENTIAL: System::out.print("ex")
			case Quantifier::UNIVERSAL: System::out.print("all")
		}
		System::out.print("[ ");
		formula.variables.forEach[ variable |
			print(variable)
			System::out.print(" ");
		]
		System::out.print("](");
		print(formula.innerFormula);
		System::out.print(")");
	}
	
	public static def dispatch void print(Function formula){
		System::out.print(formula.name + "( ");
		val int size = formula.arguments.size;
		(0..<size).forEach[index |
			print(formula.arguments.get(index));
			if((index + 1) < size){
				System::out.print(", ");
			}
		]
		System::out.print(")");
	}
	
	public static def dispatch void print(Predicate formula){
		System::out.print(formula.name + "( ");
		val int size = formula.arguments.size;
		(0..<size).forEach[index |
			print(formula.arguments.get(index));
			if((index + 1) < size){
				System::out.print(", ");
			}
		]
		System::out.print(")");
	}
	
	public static def dispatch void print(LowBindFormula formula){
		System::out.print(formula.operation.literal);
		System::out.print("(");
		print(formula.left);
		System::out.print(", ");
		print(formula.right);
	}
	
	public static def dispatch void print(NegatedFormula formula){
		System::out.print("!(");
		print(formula.innerFormula);
		System::out.print(")");
	}
	
}