package at.cl.atp.validation

import at.cl.atp.fOResolution.Variable
import at.cl.atp.fOResolution.Formula
import java.util.Set
import java.util.HashSet
import at.cl.atp.fOResolution.Constant
import at.cl.atp.fOResolution.Function
import at.cl.atp.fOResolution.Predicate
import at.cl.atp.fOResolution.LowBindFormula
import at.cl.atp.fOResolution.NegatedFormula
import at.cl.atp.fOResolution.QuantifiedFormula

class VariableChecker {
	
	def static CheckerResult check(Formula root){
		return new VariableChecker(root).check();
	}
	
	private val Formula root;
	private val Set<Variable> freeVariables;
	private val Set<VariableRebinding> reboundVariables;
	
	private new(Formula root){
		this.root = root;
		this.freeVariables = new HashSet<Variable>();
		this.reboundVariables = new HashSet<VariableRebinding>();
	}
	
	private def CheckerResult check(){
		val Set<String> emptyScope = new HashSet<String>();
		this.checkInternal(this.root, emptyScope);
		val CheckerResult result = new CheckerResult(this.freeVariables, this.reboundVariables);
		return result;
	}
	
	
	private dispatch def void checkInternal(Constant constant, Set<String> boundInScope){
		// nothing to do for constants
		return;
	}
	
	private dispatch def void checkInternal(Variable variable, Set<String> boundInScope){
		val String varname = variable.name;
		val boolean free = boundInScope.filter[scopeVar | scopeVar.contentEquals(varname)].empty;
		if(free){
			freeVariables.add(variable);	
			System::out.println("Found free variable " + variable.name);
		}
	}
	
	private dispatch def void checkInternal(QuantifiedFormula formula, Set<String> boundInScope){
		// check for rebinding
		formula.variables.forEach[variable |
			val boolean unbound = boundInScope.filter[scopeVar | scopeVar.contentEquals(variable)].empty;
			if(!unbound){
				val VariableRebinding rebinding = new VariableRebinding(formula, variable);
				reboundVariables.add(rebinding);
				System::out.println("Found rebinding of variable " + variable);
			}
		]
		// create a new scope
		val Set<String> newScope = new HashSet<String>(boundInScope);
		newScope.addAll(formula.variables);
		formula.innerFormula.checkInternal(newScope);
	}
	
	private dispatch def void checkInternal(Function function, Set<String> boundInScope){
		// check the arguments individually
		function.arguments.forEach[argument |
			argument.checkInternal(boundInScope);
		]
	}
	
	private dispatch def void checkInternal(Predicate predicate, Set<String> boundInScope){
		// check the arguments individually
		predicate.arguments.forEach[argument |
			argument.checkInternal(boundInScope);
		]
	}
	
	private dispatch def void checkInternal(LowBindFormula formula, Set<String> boundInScope){
		// check the arguments individually
		formula.left.checkInternal(boundInScope);
		formula.right.checkInternal(boundInScope);
	}
	
	private dispatch def void checkInternal(NegatedFormula negatedFormula, Set<String> boundInScope){
		// check the argument
		negatedFormula.innerFormula.checkInternal(boundInScope);
	}
	
}