package at.cl.atp.serializer;

import at.cl.atp.fOResolution.Constant;
import at.cl.atp.fOResolution.FOResolutionPackage;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.fOResolution.Variable;
import at.cl.atp.services.FOResolutionGrammarAccess;
import com.google.inject.Inject;
import com.google.inject.Provider;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractDelegatingSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("all")
public class FOResolutionSemanticSequencer extends AbstractDelegatingSemanticSequencer {

	@Inject
	private FOResolutionGrammarAccess grammarAccess;
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == FOResolutionPackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case FOResolutionPackage.CONSTANT:
				if(context == grammarAccess.getAtomRule() ||
				   context == grammarAccess.getBracedFormulaRule() ||
				   context == grammarAccess.getConstantRule() ||
				   context == grammarAccess.getHighBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaAccess().getLowBindFormulaLeftAction_1_0() ||
				   context == grammarAccess.getPredicateArgumentRule()) {
					sequence_Constant(context, (Constant) semanticObject); 
					return; 
				}
				else break;
			case FOResolutionPackage.FUNCTION:
				if(context == grammarAccess.getFunctionRule() ||
				   context == grammarAccess.getPredicateArgumentRule()) {
					sequence_Function(context, (Function) semanticObject); 
					return; 
				}
				else break;
			case FOResolutionPackage.LOW_BIND_FORMULA:
				if(context == grammarAccess.getBracedFormulaRule() ||
				   context == grammarAccess.getHighBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaAccess().getLowBindFormulaLeftAction_1_0()) {
					sequence_LowBindFormula(context, (LowBindFormula) semanticObject); 
					return; 
				}
				else break;
			case FOResolutionPackage.NEGATED_FORMULA:
				if(context == grammarAccess.getBracedFormulaRule() ||
				   context == grammarAccess.getHighBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaAccess().getLowBindFormulaLeftAction_1_0() ||
				   context == grammarAccess.getNegatedFormulaRule()) {
					sequence_NegatedFormula(context, (NegatedFormula) semanticObject); 
					return; 
				}
				else break;
			case FOResolutionPackage.PREDICATE:
				if(context == grammarAccess.getBracedFormulaRule() ||
				   context == grammarAccess.getHighBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaAccess().getLowBindFormulaLeftAction_1_0() ||
				   context == grammarAccess.getPredicateRule()) {
					sequence_Predicate(context, (Predicate) semanticObject); 
					return; 
				}
				else break;
			case FOResolutionPackage.QUANTIFIED_FORMULA:
				if(context == grammarAccess.getBracedFormulaRule() ||
				   context == grammarAccess.getHighBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaAccess().getLowBindFormulaLeftAction_1_0() ||
				   context == grammarAccess.getQuantifiedFormulaRule()) {
					sequence_QuantifiedFormula(context, (QuantifiedFormula) semanticObject); 
					return; 
				}
				else break;
			case FOResolutionPackage.VARIABLE:
				if(context == grammarAccess.getAtomRule() ||
				   context == grammarAccess.getBracedFormulaRule() ||
				   context == grammarAccess.getHighBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaRule() ||
				   context == grammarAccess.getLowBindFormulaAccess().getLowBindFormulaLeftAction_1_0() ||
				   context == grammarAccess.getPredicateArgumentRule() ||
				   context == grammarAccess.getVariableRule()) {
					sequence_Variable(context, (Variable) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     name=UPPERCASE_ID
	 */
	protected void sequence_Constant(EObject context, Constant semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, FOResolutionPackage.Literals.ATOM__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, FOResolutionPackage.Literals.ATOM__NAME));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getConstantAccess().getNameUPPERCASE_IDTerminalRuleCall_0(), semanticObject.getName());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (name=LOWERCASE_ID (arguments+=PredicateArgument arguments+=PredicateArgument*)?)
	 */
	protected void sequence_Function(EObject context, Function semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (left=LowBindFormula_LowBindFormula_1_0 operation=BinaryOperation right=LowBindFormula)
	 */
	protected void sequence_LowBindFormula(EObject context, LowBindFormula semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, FOResolutionPackage.Literals.LOW_BIND_FORMULA__LEFT) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, FOResolutionPackage.Literals.LOW_BIND_FORMULA__LEFT));
			if(transientValues.isValueTransient(semanticObject, FOResolutionPackage.Literals.LOW_BIND_FORMULA__OPERATION) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, FOResolutionPackage.Literals.LOW_BIND_FORMULA__OPERATION));
			if(transientValues.isValueTransient(semanticObject, FOResolutionPackage.Literals.LOW_BIND_FORMULA__RIGHT) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, FOResolutionPackage.Literals.LOW_BIND_FORMULA__RIGHT));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getLowBindFormulaAccess().getLowBindFormulaLeftAction_1_0(), semanticObject.getLeft());
		feeder.accept(grammarAccess.getLowBindFormulaAccess().getOperationBinaryOperationEnumRuleCall_1_1_0(), semanticObject.getOperation());
		feeder.accept(grammarAccess.getLowBindFormulaAccess().getRightLowBindFormulaParserRuleCall_1_2_0(), semanticObject.getRight());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     innerFormula=HighBindFormula
	 */
	protected void sequence_NegatedFormula(EObject context, NegatedFormula semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, FOResolutionPackage.Literals.NEGATED_FORMULA__INNER_FORMULA) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, FOResolutionPackage.Literals.NEGATED_FORMULA__INNER_FORMULA));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getNegatedFormulaAccess().getInnerFormulaHighBindFormulaParserRuleCall_1_0(), semanticObject.getInnerFormula());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (name=UPPERCASE_ID arguments+=PredicateArgument arguments+=PredicateArgument*)
	 */
	protected void sequence_Predicate(EObject context, Predicate semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (quantifier=Quantifier variables+=LOWERCASE_ID variables+=LOWERCASE_ID* innerFormula=HighBindFormula)
	 */
	protected void sequence_QuantifiedFormula(EObject context, QuantifiedFormula semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     name=LOWERCASE_ID
	 */
	protected void sequence_Variable(EObject context, Variable semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, FOResolutionPackage.Literals.ATOM__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, FOResolutionPackage.Literals.ATOM__NAME));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getVariableAccess().getNameLOWERCASE_IDTerminalRuleCall_0(), semanticObject.getName());
		feeder.finish();
	}
}
