package at.cl.atp.validation;

import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.Constant;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.fOResolution.Quantifier;
import at.cl.atp.fOResolution.Variable;
import com.google.common.base.Objects;
import java.util.Arrays;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.ExclusiveRange;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class FormulaPrettyPrinter {
  public static void _print(final Constant formula) {
    String _name = formula.getName();
    System.out.print(_name);
  }
  
  public static void _print(final Variable formula) {
    String _name = formula.getName();
    System.out.print(_name);
  }
  
  public static void _print(final QuantifiedFormula formula) {
    Quantifier _quantifier = formula.getQuantifier();
    final Quantifier _switchValue = _quantifier;
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(_switchValue,Quantifier.EXISTENTIAL)) {
        _matched=true;
        System.out.print("ex");
      }
    }
    if (!_matched) {
      if (Objects.equal(_switchValue,Quantifier.UNIVERSAL)) {
        _matched=true;
        System.out.print("all");
      }
    }
    System.out.print("[ ");
    EList<String> _variables = formula.getVariables();
    final Procedure1<String> _function = new Procedure1<String>() {
        public void apply(final String variable) {
          InputOutput.<String>print(variable);
          System.out.print(" ");
        }
      };
    IterableExtensions.<String>forEach(_variables, _function);
    System.out.print("](");
    Formula _innerFormula = formula.getInnerFormula();
    FormulaPrettyPrinter.print(_innerFormula);
    System.out.print(")");
  }
  
  public static void _print(final Function formula) {
    String _name = formula.getName();
    String _plus = (_name + "( ");
    System.out.print(_plus);
    EList<Formula> _arguments = formula.getArguments();
    final int size = _arguments.size();
    ExclusiveRange _doubleDotLessThan = new ExclusiveRange(0, size, true);
    final Procedure1<Integer> _function = new Procedure1<Integer>() {
        public void apply(final Integer index) {
          EList<Formula> _arguments = formula.getArguments();
          Formula _get = _arguments.get((index).intValue());
          FormulaPrettyPrinter.print(_get);
          int _plus = ((index).intValue() + 1);
          boolean _lessThan = (_plus < size);
          if (_lessThan) {
            System.out.print(", ");
          }
        }
      };
    IterableExtensions.<Integer>forEach(_doubleDotLessThan, _function);
    System.out.print(")");
  }
  
  public static void _print(final Predicate formula) {
    String _name = formula.getName();
    String _plus = (_name + "( ");
    System.out.print(_plus);
    EList<Formula> _arguments = formula.getArguments();
    final int size = _arguments.size();
    ExclusiveRange _doubleDotLessThan = new ExclusiveRange(0, size, true);
    final Procedure1<Integer> _function = new Procedure1<Integer>() {
        public void apply(final Integer index) {
          EList<Formula> _arguments = formula.getArguments();
          Formula _get = _arguments.get((index).intValue());
          FormulaPrettyPrinter.print(_get);
          int _plus = ((index).intValue() + 1);
          boolean _lessThan = (_plus < size);
          if (_lessThan) {
            System.out.print(", ");
          }
        }
      };
    IterableExtensions.<Integer>forEach(_doubleDotLessThan, _function);
    System.out.print(")");
  }
  
  public static void _print(final LowBindFormula formula) {
    BinaryOperation _operation = formula.getOperation();
    String _literal = _operation.getLiteral();
    System.out.print(_literal);
    System.out.print("(");
    Formula _left = formula.getLeft();
    FormulaPrettyPrinter.print(_left);
    System.out.print(", ");
    Formula _right = formula.getRight();
    FormulaPrettyPrinter.print(_right);
  }
  
  public static void _print(final NegatedFormula formula) {
    System.out.print("!(");
    Formula _innerFormula = formula.getInnerFormula();
    FormulaPrettyPrinter.print(_innerFormula);
    System.out.print(")");
  }
  
  public static void print(final Formula formula) {
    if (formula instanceof Constant) {
      _print((Constant)formula);
      return;
    } else if (formula instanceof Variable) {
      _print((Variable)formula);
      return;
    } else if (formula instanceof Function) {
      _print((Function)formula);
      return;
    } else if (formula instanceof LowBindFormula) {
      _print((LowBindFormula)formula);
      return;
    } else if (formula instanceof NegatedFormula) {
      _print((NegatedFormula)formula);
      return;
    } else if (formula instanceof Predicate) {
      _print((Predicate)formula);
      return;
    } else if (formula instanceof QuantifiedFormula) {
      _print((QuantifiedFormula)formula);
      return;
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(formula).toString());
    }
  }
}
