package at.cl.atp.validation;

import at.cl.atp.fOResolution.Constant;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.fOResolution.Variable;
import at.cl.atp.validation.CheckerResult;
import at.cl.atp.validation.VariableRebinding;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class VariableChecker {
  public static CheckerResult check(final Formula root) {
    VariableChecker _variableChecker = new VariableChecker(root);
    return _variableChecker.check();
  }
  
  private final Formula root;
  
  private final Set<Variable> freeVariables;
  
  private final Set<VariableRebinding> reboundVariables;
  
  private VariableChecker(final Formula root) {
    this.root = root;
    HashSet<Variable> _hashSet = new HashSet<Variable>();
    this.freeVariables = _hashSet;
    HashSet<VariableRebinding> _hashSet_1 = new HashSet<VariableRebinding>();
    this.reboundVariables = _hashSet_1;
  }
  
  private CheckerResult check() {
    HashSet<String> _hashSet = new HashSet<String>();
    final Set<String> emptyScope = _hashSet;
    this.checkInternal(this.root, emptyScope);
    CheckerResult _checkerResult = new CheckerResult(this.freeVariables, this.reboundVariables);
    final CheckerResult result = _checkerResult;
    return result;
  }
  
  private void _checkInternal(final Constant constant, final Set<String> boundInScope) {
    return;
  }
  
  private void _checkInternal(final Variable variable, final Set<String> boundInScope) {
    final String varname = variable.getName();
    final Function1<String,Boolean> _function = new Function1<String,Boolean>() {
        public Boolean apply(final String scopeVar) {
          boolean _contentEquals = scopeVar.contentEquals(varname);
          return Boolean.valueOf(_contentEquals);
        }
      };
    Iterable<String> _filter = IterableExtensions.<String>filter(boundInScope, _function);
    final boolean free = IterableExtensions.isEmpty(_filter);
    if (free) {
      this.freeVariables.add(variable);
      String _name = variable.getName();
      String _plus = ("Found free variable " + _name);
      System.out.println(_plus);
    }
  }
  
  private void _checkInternal(final QuantifiedFormula formula, final Set<String> boundInScope) {
    EList<String> _variables = formula.getVariables();
    final Procedure1<String> _function = new Procedure1<String>() {
        public void apply(final String variable) {
          final Function1<String,Boolean> _function = new Function1<String,Boolean>() {
              public Boolean apply(final String scopeVar) {
                boolean _contentEquals = scopeVar.contentEquals(variable);
                return Boolean.valueOf(_contentEquals);
              }
            };
          Iterable<String> _filter = IterableExtensions.<String>filter(boundInScope, _function);
          final boolean unbound = IterableExtensions.isEmpty(_filter);
          boolean _not = (!unbound);
          if (_not) {
            VariableRebinding _variableRebinding = new VariableRebinding(formula, variable);
            final VariableRebinding rebinding = _variableRebinding;
            VariableChecker.this.reboundVariables.add(rebinding);
            String _plus = ("Found rebinding of variable " + variable);
            System.out.println(_plus);
          }
        }
      };
    IterableExtensions.<String>forEach(_variables, _function);
    HashSet<String> _hashSet = new HashSet<String>(boundInScope);
    final Set<String> newScope = _hashSet;
    EList<String> _variables_1 = formula.getVariables();
    newScope.addAll(_variables_1);
    Formula _innerFormula = formula.getInnerFormula();
    this.checkInternal(_innerFormula, newScope);
  }
  
  private void _checkInternal(final Function function, final Set<String> boundInScope) {
    EList<Formula> _arguments = function.getArguments();
    final Procedure1<Formula> _function = new Procedure1<Formula>() {
        public void apply(final Formula argument) {
          VariableChecker.this.checkInternal(argument, boundInScope);
        }
      };
    IterableExtensions.<Formula>forEach(_arguments, _function);
  }
  
  private void _checkInternal(final Predicate predicate, final Set<String> boundInScope) {
    EList<Formula> _arguments = predicate.getArguments();
    final Procedure1<Formula> _function = new Procedure1<Formula>() {
        public void apply(final Formula argument) {
          VariableChecker.this.checkInternal(argument, boundInScope);
        }
      };
    IterableExtensions.<Formula>forEach(_arguments, _function);
  }
  
  private void _checkInternal(final LowBindFormula formula, final Set<String> boundInScope) {
    Formula _left = formula.getLeft();
    this.checkInternal(_left, boundInScope);
    Formula _right = formula.getRight();
    this.checkInternal(_right, boundInScope);
  }
  
  private void _checkInternal(final NegatedFormula negatedFormula, final Set<String> boundInScope) {
    Formula _innerFormula = negatedFormula.getInnerFormula();
    this.checkInternal(_innerFormula, boundInScope);
  }
  
  private void checkInternal(final Formula constant, final Set<String> boundInScope) {
    if (constant instanceof Constant) {
      _checkInternal((Constant)constant, boundInScope);
      return;
    } else if (constant instanceof Variable) {
      _checkInternal((Variable)constant, boundInScope);
      return;
    } else if (constant instanceof Function) {
      _checkInternal((Function)constant, boundInScope);
      return;
    } else if (constant instanceof LowBindFormula) {
      _checkInternal((LowBindFormula)constant, boundInScope);
      return;
    } else if (constant instanceof NegatedFormula) {
      _checkInternal((NegatedFormula)constant, boundInScope);
      return;
    } else if (constant instanceof Predicate) {
      _checkInternal((Predicate)constant, boundInScope);
      return;
    } else if (constant instanceof QuantifiedFormula) {
      _checkInternal((QuantifiedFormula)constant, boundInScope);
      return;
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(constant, boundInScope).toString());
    }
  }
}
