package at.cl.atp.foresolution.model.container;

import at.cl.atp.foresolution.alphabet.Alphabet;
import at.cl.atp.foresolution.alphabet.GreekAlphabet;
import at.cl.atp.foresolution.alphabet.LatinAlphabet;

public class SkolemNameProvider {

	private int variableIndex = 0;
	private int functionIndex = 0;
	private int predicateIndex = 0;

	private Alphabet variableAlphabet;
	private Alphabet functionAlphabet;
	private Alphabet predicateAlphabet;

	public SkolemNameProvider(final String variableBase, final String functionBase, final String predicateBase) {
		this.variableAlphabet = GreekAlphabet.instance;
		this.functionAlphabet = LatinAlphabet.instance;
		this.predicateAlphabet = LatinAlphabet.instance;
	}

	public SkolemNameProvider() {
		this("v", "f", "P");
	}

	public String nextSkolemVariable() {
		int letterIndex = this.variableIndex % this.variableAlphabet.getAlphabetSize();
		int letterSubscript = this.variableIndex / this.variableAlphabet.getAlphabetSize();
		char character = this.variableAlphabet.getUnicodeLower(letterIndex);
		String name = "_" + character + "" + letterSubscript;
		this.variableIndex++;
		return name;
	}

	public String nextSkolemFunction() {
		int letterIndex = this.functionIndex % this.functionAlphabet.getAlphabetSize();
		int letterSubscript = this.functionIndex / this.functionAlphabet.getAlphabetSize();
		char character = this.functionAlphabet.getUnicodeLower(letterIndex);
		String name = "_" + character + "" + letterSubscript;
		this.functionIndex++;
		return name;
	}

	public String nextSkolemPredicate() {
		int letterIndex = this.predicateIndex % this.predicateAlphabet.getAlphabetSize();
		int letterSubscript = this.predicateIndex / this.predicateAlphabet.getAlphabetSize();
		char character = this.predicateAlphabet.getUnicodeUpper(letterIndex);
		String name = "_" + character + "" + letterSubscript;
		this.predicateIndex++;
		return name;
	}
}
