package at.cl.atp.foresolution.settings;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

public class Settings {

	private static final Properties PROPERTIES;

	public static final String SETTING_GREEK_LETTERS_IN_OUTPUT = "console.useGreek";
	public static final String SETTING_USE_FACTORIZATION = "resolution.useFactorization";
	public static final String SETTING_NO_RENAME = "preprocessing.noRenaming";
	public static final String SETTING_NEGATE_INPUT = "preprocessing.negateInput";

	private static final String DEFAULT_GREEK_LETTERS_IN_OUTPUT = "true";
	private static final String DEFAULT_USE_FACTORIZATION = "true";
	private static final String DEFAULT_NO_RENAME = "false";
	private static final String DEFAULT_NEGATE_INPUT = "true";

	static {
		PROPERTIES = new Properties();
		File settingsFile = new File("settings.ini");
		try {
			if (settingsFile.exists() == false) {
				settingsFile.createNewFile();
			}
			FileInputStream fis = new FileInputStream(settingsFile);
			PROPERTIES.load(fis);
			fis.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (PROPERTIES.containsKey(SETTING_GREEK_LETTERS_IN_OUTPUT) == false) {
			PROPERTIES.put(SETTING_GREEK_LETTERS_IN_OUTPUT, DEFAULT_GREEK_LETTERS_IN_OUTPUT);
		}
		if (PROPERTIES.containsKey(SETTING_NO_RENAME) == false) {
			PROPERTIES.put(SETTING_NO_RENAME, DEFAULT_NO_RENAME);
		}
		if (PROPERTIES.containsKey(SETTING_NEGATE_INPUT) == false) {
			PROPERTIES.put(SETTING_NEGATE_INPUT, DEFAULT_NEGATE_INPUT);
		}
		if (PROPERTIES.containsKey(SETTING_USE_FACTORIZATION) == false) {
			PROPERTIES.put(SETTING_USE_FACTORIZATION, DEFAULT_USE_FACTORIZATION);
		}
		try {
			FileOutputStream fos = new FileOutputStream(settingsFile);
			PROPERTIES.store(fos, "");
			fos.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static final Map<String, String> TEMP_SETTINGS = new HashMap<String, String>();

	private static boolean readBooleanProperty(final String propertyName) {
		String property = null;
		if (TEMP_SETTINGS.containsKey(propertyName)) {
			property = TEMP_SETTINGS.get(propertyName);
		}
		if (property == null) {
			property = (String) PROPERTIES.get(propertyName);
		}
		return property.trim().contentEquals("true");
	}

	public static boolean greekLettersInOutput() {
		return readBooleanProperty(SETTING_GREEK_LETTERS_IN_OUTPUT);
	}

	public static boolean noRenaming() {
		return readBooleanProperty(SETTING_NO_RENAME);
	}

	public static boolean negateInput() {
		return readBooleanProperty(SETTING_NEGATE_INPUT);
	}

	public static boolean useFactorization() {
		return readBooleanProperty(SETTING_USE_FACTORIZATION);
	}

	public static void setTemporaryProperty(final String name, final boolean value) {
		if (value) {
			TEMP_SETTINGS.put(name, "true");
		} else {
			TEMP_SETTINGS.put(name, "false");
		}
	}

	public static void clearTemporarySettings() {
		TEMP_SETTINGS.clear();
	}
}
