package at.cl.atp.foresolution.tptp.converter;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.FOResolutionFactory;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.fOResolution.Quantifier;
import at.cl.atp.fOResolution.Variable;
import at.cl.atp.tPTP.Annotated_formula;
import at.cl.atp.tPTP.CNF_annotated;
import at.cl.atp.tPTP.FOF_Formula;
import at.cl.atp.tPTP.TPTP_File;

public class TPTPConverter {

	// STATIC-----------------------------------------------------------
	public static Formula convert(final TPTP_File tptpFile) {
		return new TPTPConverter(tptpFile).doConvert();
	}

	// END STATIC ------------------------------------------------------

	private final TPTP_File input;
	private final TPTPtoFormulaSwitch tptpSwitch;
	private final Set<Formula> hypothesis;
	private final Set<Formula> conjectures;

	private TPTPConverter(final TPTP_File file) {
		this.input = file;
		this.tptpSwitch = new TPTPtoFormulaSwitch();
		this.hypothesis = new HashSet<Formula>();
		this.conjectures = new HashSet<Formula>();
	}

	// private Formula doConvert() {
	// for (Annotated_formula annotatedFormula : this.input.getInputs()) {
	// if (annotatedFormula instanceof CNF_annotated) {
	// CNF_annotated cnfAnnotated = (CNF_annotated) annotatedFormula;
	// Formula formula = this.convertFormula(cnfAnnotated.getFormula());
	// switch (cnfAnnotated.getRole()) {
	// case CONJECTURE:
	// this.conjectures.add(formula);
	// break;
	// case HYPOTHESIS:
	// this.hypothesis.add(formula);
	// break;
	// case NEGATED_CONJECTURE:
	// NegatedFormula negation =
	// FOResolutionFactory.eINSTANCE.createNegatedFormula();
	// negation.setInnerFormula(formula);
	// this.conjectures.add(negation);
	// break;
	// }
	// } else {
	// // TODO if other TPTP formula types are added, add them here too
	// }
	// }
	// // assemble the parts into a single, large implication
	// LowBindFormula rootImplication =
	// FOResolutionFactory.eINSTANCE.createLowBindFormula();
	// rootImplication.setOperation(BinaryOperation.IMPLICATION);
	// if (this.hypothesis.size() == 0) {
	// // left hand side is a tautology
	// rootImplication.setLeft(this.createTautology());
	// } else if (this.hypothesis.size() == 1) {
	// rootImplication.setLeft(this.hypothesis.iterator().next());
	// } else {
	// rootImplication.setLeft(this.constructMultivaluedConjunction(this.hypothesis.iterator()));
	// }
	// if (this.conjectures.size() == 0) {
	// throw new RuntimeException("The conjecture is empty!");
	// } else if (this.conjectures.size() == 1) {
	// rootImplication.setRight(this.conjectures.iterator().next());
	// } else {
	// rootImplication.setRight(this.constructMultivaluedConjunction(this.conjectures.iterator()));
	// }
	// return rootImplication;
	// }

	private Formula doConvert() {
		Set<Formula> allFormulas = new HashSet<Formula>();
		for (Annotated_formula annotatedFormula : this.input.getInputs()) {
			if (annotatedFormula instanceof CNF_annotated) {
				CNF_annotated cnfAnnotated = (CNF_annotated) annotatedFormula;
				Formula formula = this.convertFormula(cnfAnnotated.getFormula());
				switch (cnfAnnotated.getRole()) {
				case CONJECTURE:
					this.conjectures.add(formula);
					break;
				case HYPOTHESIS:
					this.hypothesis.add(formula);
					break;
				case NEGATED_CONJECTURE:
					this.conjectures.add(formula);
					break;
				}
			} else {
				// TODO if other TPTP formula types are added, add them here too
			}
		}
		allFormulas.addAll(this.conjectures);
		allFormulas.addAll(this.hypothesis);
		LowBindFormula root = FOResolutionFactory.eINSTANCE.createLowBindFormula();
		root.setOperation(BinaryOperation.CONJUNCTION);
		Iterator<Formula> iterator = allFormulas.iterator();
		root.setLeft(iterator.next());
		root.setRight(this.constructMultivaluedConjunction(iterator));
		return root;
	}

	private Formula constructMultivaluedConjunction(final Iterator<Formula> iterator) {
		LowBindFormula formula = FOResolutionFactory.eINSTANCE.createLowBindFormula();
		formula.setOperation(BinaryOperation.CONJUNCTION);
		formula.setLeft(iterator.next());
		if (iterator.hasNext()) {
			this.constructMultivaluedConjunctionRec(iterator, formula);
		}
		return formula;
	}

	private void constructMultivaluedConjunctionRec(final Iterator<Formula> iterator, final LowBindFormula parent) {
		Formula nextConjunct = iterator.next();
		if (iterator.hasNext()) {
			LowBindFormula conjunction = FOResolutionFactory.eINSTANCE.createLowBindFormula();
			conjunction.setOperation(BinaryOperation.CONJUNCTION);
			conjunction.setLeft(nextConjunct);
			parent.setRight(conjunction);
			this.constructMultivaluedConjunctionRec(iterator, conjunction);
		} else {
			parent.setRight(nextConjunct);
		}
	}

	private Formula convertFormula(final FOF_Formula fofFormula) {
		Formula result = this.tptpSwitch.doSwitch(fofFormula);
		return result;
	}

	private Formula createTautology() {
		QuantifiedFormula formula = FOResolutionFactory.eINSTANCE.createQuantifiedFormula();
		formula.setQuantifier(Quantifier.UNIVERSAL);
		LowBindFormula disjunction = FOResolutionFactory.eINSTANCE.createLowBindFormula();
		disjunction.setOperation(BinaryOperation.DISJUNCTION);
		Variable x1 = FOResolutionFactory.eINSTANCE.createVariable();
		Predicate p1 = FOResolutionFactory.eINSTANCE.createPredicate();
		p1.setName("P");
		p1.getArguments().add(x1);
		Variable x2 = FOResolutionFactory.eINSTANCE.createVariable();
		Predicate p2 = FOResolutionFactory.eINSTANCE.createPredicate();
		p2.setName("P");
		p2.getArguments().add(x2);
		NegatedFormula negation = FOResolutionFactory.eINSTANCE.createNegatedFormula();
		negation.setInnerFormula(p2);
		disjunction.setLeft(p1);
		disjunction.setRight(negation);
		formula.setInnerFormula(disjunction);
		return formula;
	}
}
