package at.cl.atp.foresolution.tptp.converter;

import org.eclipse.emf.ecore.EObject;

import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.Constant;
import at.cl.atp.fOResolution.FOResolutionFactory;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.fOResolution.Quantifier;
import at.cl.atp.fOResolution.Variable;
import at.cl.atp.tPTP.Annotated_formula;
import at.cl.atp.tPTP.CNF_annotated;
import at.cl.atp.tPTP.FOF_Atom;
import at.cl.atp.tPTP.FOF_Constant;
import at.cl.atp.tPTP.FOF_Formula;
import at.cl.atp.tPTP.FOF_Function;
import at.cl.atp.tPTP.FOF_NegatedFormula;
import at.cl.atp.tPTP.FOF_Predicate;
import at.cl.atp.tPTP.FOF_QuantifiedFormula;
import at.cl.atp.tPTP.FOF_Variable;
import at.cl.atp.tPTP.TPTP_File;
import at.cl.atp.tPTP.util.TPTPSwitch;

public class TPTPtoFormulaSwitch extends TPTPSwitch<Formula> {

	@Override
	public Formula caseTPTP_File(final TPTP_File file) {
		return null;
	}

	@Override
	public Formula caseAnnotated_formula(final Annotated_formula object) {
		return null;
	}

	@Override
	public Formula caseCNF_annotated(final CNF_annotated object) {
		return null;
	}

	@Override
	public Formula caseFOF_Formula(final FOF_Formula object) {
		if (object.getOperation() != null) {
			BinaryOperation op = null;
			switch (object.getOperation()) {
			case CONJUNCTION:
				op = BinaryOperation.CONJUNCTION;
				break;
			case DISJUNCTION:
				op = BinaryOperation.DISJUNCTION;
				break;
			case IMPLICATION:
				op = BinaryOperation.IMPLICATION;
				break;
			}
			LowBindFormula formula = FOResolutionFactory.eINSTANCE.createLowBindFormula();
			formula.setOperation(op);
			FOF_Formula fof_left = object.getLeft();
			FOF_Formula fof_right = object.getRight();
			Formula left = this.doSwitch(fof_left);
			Formula right = this.doSwitch(fof_right);
			formula.setLeft(left);
			formula.setRight(right);
			return formula;
		} else {
			return this.doSwitch(object.getLeft());
		}
	}

	@Override
	public Formula caseFOF_QuantifiedFormula(final FOF_QuantifiedFormula object) {
		QuantifiedFormula result = FOResolutionFactory.eINSTANCE.createQuantifiedFormula();
		result.getVariables().addAll(object.getVariables());
		switch (object.getQuantifier()) {
		case EXISTENTIAL:
			result.setQuantifier(Quantifier.EXISTENTIAL);
			break;
		case UNIVERSAL:
			result.setQuantifier(Quantifier.UNIVERSAL);
		}
		FOF_Formula fof_inner = object.getInnerFormula();
		result.setInnerFormula(this.doSwitch(fof_inner));
		return result;
	}

	@Override
	public Formula caseFOF_NegatedFormula(final FOF_NegatedFormula object) {
		NegatedFormula result = FOResolutionFactory.eINSTANCE.createNegatedFormula();
		FOF_Formula fof_inner = object.getInnerFormula();
		result.setInnerFormula(this.doSwitch(fof_inner));
		return result;
	}

	@Override
	public Formula caseFOF_Predicate(final FOF_Predicate object) {
		Predicate result = FOResolutionFactory.eINSTANCE.createPredicate();
		result.setName(object.getName());
		for (FOF_Formula argument : object.getArguments()) {
			result.getArguments().add(this.doSwitch(argument));
		}
		return result;
	}

	@Override
	public Formula caseFOF_Function(final FOF_Function object) {
		Function result = FOResolutionFactory.eINSTANCE.createFunction();
		result.setName(object.getName());
		for (FOF_Formula argument : object.getArguments()) {
			result.getArguments().add(this.doSwitch(argument));
		}
		return result;
	}

	@Override
	public Formula caseFOF_Atom(final FOF_Atom object) {
		return null;
	}

	@Override
	public Formula caseFOF_Variable(final FOF_Variable object) {
		Variable result = FOResolutionFactory.eINSTANCE.createVariable();
		result.setName(object.getName());
		return result;
	}

	@Override
	public Formula caseFOF_Constant(final FOF_Constant object) {
		Constant result = FOResolutionFactory.eINSTANCE.createConstant();
		result.setName(object.getName());
		return result;
	}

	@Override
	public Formula defaultCase(final EObject object) {
		throw new RuntimeException("Unknown class: " + object.getClass().getName());
	}
}
