package at.cl.atp.foresolution.transformations;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.ecore.util.EcoreUtil;

import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.FOResolutionFactory;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.foresolution.model.container.FormulaContainer;

public class ClausalFormTransformation implements FormulaTransformation {

	public static final ClausalFormTransformation instance;

	static {
		instance = new ClausalFormTransformation();
	}

	private ClausalFormTransformation() {
	}

	private boolean changed = false;

	@Override
	public Set<TransformationType> getRequiredTransformations() {
		Set<TransformationType> requirements = new HashSet<TransformationType>();
		requirements.add(TransformationType.RECTIFICATION);
		requirements.add(TransformationType.IMPLICATION_ELIMINATION);
		requirements.add(TransformationType.NEGATION_NORMAL_FORM);
		requirements.add(TransformationType.STRUCTURAL_SKOLEMIZATION);
		return requirements;
	}

	@Override
	public void transform(final FormulaContainer formula) {
		if (formula.getAppliedTransformations().contains(TransformationType.CLAUSAL_FORM)) {
			return;
		}
		TransformationProvider.resolvePreconditions(this, formula);
		this.changed = true;
		while (this.changed) {
			this.changed = false;
			formula.setFormula(this.transformRec(formula.getFormula()));
		}
	}

	private Formula transformRec(final Formula formula) {
		if (formula instanceof QuantifiedFormula) {
			QuantifiedFormula f = (QuantifiedFormula) formula;
			return this.transformRec(f.getInnerFormula());
		} else if (formula instanceof LowBindFormula) {
			LowBindFormula f = (LowBindFormula) formula;
			if (f.getOperation() == BinaryOperation.DISJUNCTION) {
				if (this.isConjunction(f.getLeft())) {
					// case (P and Q) or R --> (P or R) and (Q or R)
					LowBindFormula leftConjunction = (LowBindFormula) f.getLeft();
					Formula p = leftConjunction.getLeft();
					Formula q = leftConjunction.getRight();
					Formula r = f.getRight();
					LowBindFormula newAnd = FOResolutionFactory.eINSTANCE.createLowBindFormula();
					newAnd.setOperation(BinaryOperation.CONJUNCTION);
					LowBindFormula leftOr = FOResolutionFactory.eINSTANCE.createLowBindFormula();
					leftOr.setOperation(BinaryOperation.DISJUNCTION);
					LowBindFormula rightOr = FOResolutionFactory.eINSTANCE.createLowBindFormula();
					rightOr.setOperation(BinaryOperation.DISJUNCTION);
					leftOr.setLeft(p);
					leftOr.setRight(r);
					rightOr.setLeft(q);
					rightOr.setRight(EcoreUtil.copy(r));
					newAnd.setLeft(leftOr);
					newAnd.setRight(rightOr);
					this.changed = true;
					return newAnd;
				} else if (this.isConjunction(f.getRight())) {
					// case P or (Q and R) --> (P or Q) and (P or R)
					LowBindFormula rightConjunction = (LowBindFormula) f.getRight();
					Formula p = f.getLeft();
					Formula q = rightConjunction.getLeft();
					Formula r = rightConjunction.getRight();
					LowBindFormula newAnd = FOResolutionFactory.eINSTANCE.createLowBindFormula();
					newAnd.setOperation(BinaryOperation.CONJUNCTION);
					LowBindFormula leftOr = FOResolutionFactory.eINSTANCE.createLowBindFormula();
					leftOr.setOperation(BinaryOperation.DISJUNCTION);
					LowBindFormula rightOr = FOResolutionFactory.eINSTANCE.createLowBindFormula();
					rightOr.setOperation(BinaryOperation.DISJUNCTION);
					leftOr.setLeft(p);
					leftOr.setRight(q);
					rightOr.setLeft(EcoreUtil.copy(p));
					rightOr.setRight(r);
					newAnd.setLeft(leftOr);
					newAnd.setRight(rightOr);
					this.changed = true;
					return newAnd;
				} else {
					f.setLeft(this.transformRec(f.getLeft()));
					f.setRight(this.transformRec(f.getRight()));
					return f;
				}
			} else {
				f.setLeft(this.transformRec(f.getLeft()));
				f.setRight(this.transformRec(f.getRight()));
				return f;
			}
		}
		return formula;
	}

	private boolean isConjunction(final Formula formula) {
		if (formula instanceof LowBindFormula) {
			return ((LowBindFormula) formula).getOperation() == BinaryOperation.CONJUNCTION;
		}
		return false;
	}

}
