package at.cl.atp.foresolution.transformations;

import java.util.Collections;
import java.util.Set;

import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.FOResolutionFactory;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.foresolution.model.container.FormulaContainer;

public class ImplicationEliminationTransformation implements FormulaTransformation {

	public static final ImplicationEliminationTransformation instance;

	static {
		instance = new ImplicationEliminationTransformation();
	}

	private ImplicationEliminationTransformation() {

	}

	@Override
	public Set<TransformationType> getRequiredTransformations() {
		return Collections.emptySet();
	}

	@Override
	public void transform(final FormulaContainer formula) {
		if (formula.getAppliedTransformations().contains(TransformationType.IMPLICATION_ELIMINATION)) {
			return;
		}
		TransformationProvider.resolvePreconditions(this, formula);
		formula.setFormula(instance.doTransform(formula.getFormula()));
		formula.getAppliedTransformations().add(TransformationType.IMPLICATION_ELIMINATION);
	}

	private Formula doTransform(final Formula formula) {
		if (formula instanceof LowBindFormula) {
			LowBindFormula f = (LowBindFormula) formula;
			if (f.getOperation() == BinaryOperation.IMPLICATION) {
				// turn the implication into a disjunction
				// and negate the left argument
				f.setOperation(BinaryOperation.DISJUNCTION);
				NegatedFormula negation = FOResolutionFactory.eINSTANCE.createNegatedFormula();
				negation.setInnerFormula(f.getLeft());
				f.setLeft(negation);
			}
			// apply to both arguments
			this.doTransform(f.getLeft());
			this.doTransform(f.getRight());
		} else if (formula instanceof QuantifiedFormula) {
			this.doTransform(((QuantifiedFormula) formula).getInnerFormula());
		} else if (formula instanceof NegatedFormula) {
			this.doTransform(((NegatedFormula) formula).getInnerFormula());
		}
		// note: we do not care about predicates and functions here
		// as there cannot be an implication inside those.
		return formula;
	}

}
