package at.cl.atp.foresolution.transformations;

import java.util.HashMap;
import java.util.Set;

import at.cl.atp.foresolution.model.container.FormulaContainer;

public class TransformationProvider {

	private static final HashMap<TransformationType, FormulaTransformation> transformationBindings;

	static {
		transformationBindings = new HashMap<TransformationType, FormulaTransformation>();
		transformationBindings.put(TransformationType.IMPLICATION_ELIMINATION,
				ImplicationEliminationTransformation.instance);
		transformationBindings.put(TransformationType.NEGATION_NORMAL_FORM, NNFTransformation.instance);
		transformationBindings.put(TransformationType.RECTIFICATION, RectificationTransformation.instance);
		transformationBindings.put(TransformationType.STRUCTURAL_SKOLEMIZATION, StructuralSkolemization.instance);
		transformationBindings.put(TransformationType.CLAUSAL_FORM, ClausalFormTransformation.instance);
	}

	public static FormulaTransformation getTransformation(final TransformationType type) {
		return transformationBindings.get(type);
	}

	public static void resolvePreconditions(final FormulaTransformation transformation, final FormulaContainer formula) {
		Set<TransformationType> requirements = transformation.getRequiredTransformations();
		for (TransformationType transformationType : requirements) {
			getTransformation(transformationType).transform(formula);
		}
	}
}
