package at.cl.atp.foresolution.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import at.cl.atp.fOResolution.Atom;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.foresolution.alphabet.GreekAlphabet;
import at.cl.atp.foresolution.settings.Settings;

public class FormulaFormatter {

	public static String format(final Formula formula) {
		StringBuilder builder = new StringBuilder();
		formatRec(formula, builder);
		return builder.toString();
	}

	private static void formatRec(final Formula formula, final StringBuilder builder) {
		if (formula instanceof Atom) {
			builder.append(formatIdentifier(((Atom) formula).getName()));
		} else if (formula instanceof QuantifiedFormula) {
			QuantifiedFormula f = (QuantifiedFormula) formula;
			builder.append(f.getQuantifier().getLiteral());
			builder.append("[");
			int size = f.getVariables().size();
			for (int i = 0; i < size; i++) {
				builder.append(formatIdentifier(f.getVariables().get(i)));
				if ((i + 1) < size) {
					builder.append(",");
				}
			}
			builder.append("](");
			formatRec(f.getInnerFormula(), builder);
			builder.append(")");
		} else if (formula instanceof LowBindFormula) {
			LowBindFormula f = (LowBindFormula) formula;
			builder.append(f.getOperation().getLiteral());
			builder.append("(");
			formatRec(f.getLeft(), builder);
			builder.append(", ");
			formatRec(f.getRight(), builder);
			builder.append(")");
		} else if (formula instanceof NegatedFormula) {
			builder.append("!(");
			formatRec(((NegatedFormula) formula).getInnerFormula(), builder);
			builder.append(")");
		} else if (formula instanceof Predicate) {
			Predicate f = (Predicate) formula;
			builder.append(formatIdentifier(f.getName()));
			builder.append("(");
			int argumentCount = f.getArguments().size();
			for (int index = 0; index < argumentCount; index++) {
				Formula argument = f.getArguments().get(index);
				formatRec(argument, builder);
				if ((index + 1) < argumentCount) {
					builder.append(", ");
				}
			}
			builder.append(")");
		} else if (formula instanceof Function) {
			Function f = (Function) formula;
			builder.append(formatIdentifier(f.getName()));
			builder.append("(");
			int argumentCount = f.getArguments().size();
			for (int index = 0; index < argumentCount; index++) {
				Formula argument = f.getArguments().get(index);
				formatRec(argument, builder);
				if ((index + 1) < argumentCount) {
					builder.append(", ");
				}
			}
			builder.append(")");
		}
	}

	private static String formatIdentifier(final String identifier) {
		String result = identifier;
		if (result.startsWith("_")) {
			// cut leading underscore
			result = result.substring(1);
		}
		Pattern numberPattern = Pattern.compile("[0-9]");
		Matcher matcher = numberPattern.matcher(result);
		if (matcher.find()) {
			int index = Integer.parseInt(matcher.group());
			if (index == 0) {
				// cut trailing index 0
				result = result.substring(result.length() - 2, result.length() - 1);
			}
		}
		if (Settings.greekLettersInOutput()) {
			return result;
		} else {
			// convert to latin
			StringBuilder builder = new StringBuilder();
			for (char character : result.toCharArray()) {
				if (GreekAlphabet.instance.contains(character)) {
					builder.append(GreekAlphabet.instance.toLatex(character));
				} else {
					builder.append(character);
				}
			}
			return builder.toString();
		}
	}
}
