package at.cl.atp.foresolution.model.clause;

import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.foresolution.model.container.FormulaContainer;
import at.cl.atp.foresolution.transformations.NormalFormTransformations;
import at.cl.atp.foresolution.transformations.TransformationType;

public class ClausalFormBuilder {

	public static ClausalForm build(final FormulaContainer input) {
		if (input.getAppliedTransformations().contains(TransformationType.CLAUSAL_FORM) == false) {
			NormalFormTransformations.toClausalForm(input);
		}
		ClausalForm clausalForm = new ClausalForm();
		buildClausesRec(input.getFormula(), clausalForm);
		return clausalForm;
	}

	private static void buildClausesRec(final Formula formula, final ClausalForm clausalForm) {
		if (formula instanceof LowBindFormula) {
			LowBindFormula f = (LowBindFormula) formula;
			if (f.getOperation() == BinaryOperation.CONJUNCTION) {
				buildClausesRec(f.getLeft(), clausalForm);
				buildClausesRec(f.getRight(), clausalForm);
				return;
			}
		}
		clausalForm.add(buildClause(formula));
	}

	private static Clause buildClause(final Formula formula) {
		Clause clause = new Clause();
		buildClauseRec(formula, clause);
		return clause;
	}

	private static void buildClauseRec(final Formula formula, final Clause clause) {
		if (formula instanceof LowBindFormula) {
			LowBindFormula f = (LowBindFormula) formula;
			if (f.getOperation() == BinaryOperation.DISJUNCTION) {
				buildClauseRec(f.getLeft(), clause);
				buildClauseRec(f.getRight(), clause);
				return;
			}
		}
		ClauseLiteral literal = new ClauseLiteral(formula);
		clause.addLiteral(literal);
	}
}
