package at.cl.atp.foresolution.util;

import at.cl.atp.fOResolution.Atom;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;

public class FormulaFormatter {

	public static String format(final Formula formula) {
		StringBuilder builder = new StringBuilder();
		formatRec(formula, builder);
		return builder.toString();
	}

	private static void formatRec(final Formula formula, final StringBuilder builder) {
		if (formula instanceof Atom) {
			builder.append(((Atom) formula).getName());
		} else if (formula instanceof QuantifiedFormula) {
			QuantifiedFormula f = (QuantifiedFormula) formula;
			builder.append(f.getQuantifier().getLiteral());
			builder.append("[");
			int size = f.getVariables().size();
			for (int i = 0; i < size; i++) {
				builder.append(f.getVariables().get(i));
				if ((i + 1) < size) {
					builder.append(",");
				}
			}
			builder.append("](");
			formatRec(f.getInnerFormula(), builder);
			builder.append(")");
		} else if (formula instanceof LowBindFormula) {
			LowBindFormula f = (LowBindFormula) formula;
			builder.append(f.getOperation().getLiteral());
			builder.append("(");
			formatRec(f.getLeft(), builder);
			builder.append(", ");
			formatRec(f.getRight(), builder);
			builder.append(")");
		} else if (formula instanceof NegatedFormula) {
			builder.append("!(");
			formatRec(((NegatedFormula) formula).getInnerFormula(), builder);
			builder.append(")");
		} else if (formula instanceof Predicate) {
			Predicate f = (Predicate) formula;
			builder.append(f.getName());
			builder.append("(");
			int argumentCount = f.getArguments().size();
			for (int index = 0; index < argumentCount; index++) {
				Formula argument = f.getArguments().get(index);
				formatRec(argument, builder);
				if ((index + 1) < argumentCount) {
					builder.append(", ");
				}
			}
			builder.append(")");
		} else if (formula instanceof Function) {
			Function f = (Function) formula;
			builder.append(f.getName());
			builder.append("(");
			int argumentCount = f.getArguments().size();
			for (int index = 0; index < argumentCount; index++) {
				Formula argument = f.getArguments().get(index);
				formatRec(argument, builder);
				if ((index + 1) < argumentCount) {
					builder.append(", ");
				}
			}
			builder.append(")");
		}
	}
}
