package clash;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Set;

import org.junit.Test;

import util.TestHelper;
import at.cl.atp.fOResolution.Constant;
import at.cl.atp.fOResolution.FOResolutionFactory;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.Variable;
import at.cl.atp.foresolution.exceptions.ClashProducedBoxException;
import at.cl.atp.foresolution.model.clause.ClausalForm;
import at.cl.atp.foresolution.model.clause.ClausalFormBuilder;
import at.cl.atp.foresolution.model.clause.Clause;
import at.cl.atp.foresolution.model.clause.ClauseLiteral;
import at.cl.atp.foresolution.model.container.FormulaContainer;

public class ClashTest {

	@Test
	public void clashTest01() {
		for (int i = 0; i < 100; i++) {
			Formula formula = TestHelper.load("clashTest1");
			ClausalForm clausalForm = ClausalFormBuilder.build(new FormulaContainer(formula));
			Clause first = null;
			Clause second = null;
			for (Clause clause : clausalForm.getClauses()) {
				if (first == null) {
					first = clause;
				} else if (second == null) {
					second = clause;
				} else {
					fail();
				}
			}
			Set<Clause> resolvents = first.clash(second);
			assertTrue(resolvents != null && resolvents.size() == 1);
		}

	}

	@Test
	public void clashTest02() {
		Variable x = FOResolutionFactory.eINSTANCE.createVariable();
		x.setName("x");
		Predicate p1 = FOResolutionFactory.eINSTANCE.createPredicate();
		p1.setName("P");
		p1.getArguments().add(x);
		Clause clause = Clause.createUnitClause(new ClauseLiteral(p1));

		Variable y = FOResolutionFactory.eINSTANCE.createVariable();
		y.setName("y");
		Predicate p2 = FOResolutionFactory.eINSTANCE.createPredicate();
		p2.setName("P");
		p2.getArguments().add(y);
		NegatedFormula negation = FOResolutionFactory.eINSTANCE.createNegatedFormula();
		negation.setInnerFormula(p2);
		Clause negatedClause = Clause.createUnitClause(new ClauseLiteral(negation));
		System.out.println(clause.toString());
		System.out.println(negatedClause.toString());
		try {
			@SuppressWarnings("unused")
			Set<Clause> clashResult = clause.clash(negatedClause);
			fail();
		} catch (ClashProducedBoxException e) {
			System.out.println(e.getMessage());
		}
		clause.clearClashCache();
		negatedClause.clearClashCache();
		System.out.println(clause.toString());
		System.out.println(negatedClause.toString());
		try {
			@SuppressWarnings("unused")
			Set<Clause> clashResult = negatedClause.clash(clause);
			fail("Could not reach BOX!");
		} catch (ClashProducedBoxException e) {
			System.out.println(e.getMessage());
		}
	}

	@Test
	public void clashTest03() {
		Variable x = FOResolutionFactory.eINSTANCE.createVariable();
		x.setName("x");
		Predicate p1 = FOResolutionFactory.eINSTANCE.createPredicate();
		p1.setName("P");
		p1.getArguments().add(x);
		Clause clause = Clause.createUnitClause(new ClauseLiteral(p1));

		Constant a = FOResolutionFactory.eINSTANCE.createConstant();
		a.setName("A");
		Predicate p2 = FOResolutionFactory.eINSTANCE.createPredicate();
		p2.setName("P");
		p2.getArguments().add(a);
		NegatedFormula negation = FOResolutionFactory.eINSTANCE.createNegatedFormula();
		negation.setInnerFormula(p2);
		Clause negatedClause = Clause.createUnitClause(new ClauseLiteral(negation));
		System.out.println(clause.toString());
		System.out.println(negatedClause.toString());
		try {
			@SuppressWarnings("unused")
			Set<Clause> clashResult = clause.clash(negatedClause);
			fail();
		} catch (ClashProducedBoxException e) {
			System.out.println(e.getMessage());
		}
		clause.clearClashCache();
		negatedClause.clearClashCache();
		System.out.println(clause.toString());
		System.out.println(negatedClause.toString());
		try {
			@SuppressWarnings("unused")
			Set<Clause> clashResult = negatedClause.clash(clause);
			fail("Could not reach BOX!");
		} catch (ClashProducedBoxException e) {
			System.out.println(e.getMessage());
		}
	}

}
