package resolution;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.Scanner;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import util.TestHelper;
import at.cl.atp.foresolution.prover.ResolutionOutput;
import at.cl.atp.foresolution.prover.ResolutionResult;
import at.cl.atp.foresolution.settings.Settings;

public class ResolutionTest {

	@BeforeClass
	public static void buildUp() {
		Settings.setTemporaryProperty(Settings.SETTING_NEGATE_INPUT, true);
		Settings.setTemporaryProperty(Settings.SETTING_NO_RENAME, false);
		Settings.setTemporaryProperty(Settings.SETTING_USE_FACTORIZATION, false);
	}

	@AfterClass
	public static void tearDown() {
		Settings.clearTemporarySettings();
	}

	@Test
	public void basicTest01() {
		ResolutionResult result = TestHelper.executeResolution("basicTest").getResult();
		System.out.println();
		System.out.println();
		assertEquals("Basic example failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void killerExampleTest() {
		ResolutionResult result = TestHelper.executeResolution("killerExample").getResult();
		System.out.println();
		System.out.println();
		assertEquals("Killer Example failed!", ResolutionResult.BOX, result);
	}

	@Ignore
	@Test
	public void example01Times10000Test() {
		int successful = 0;
		int failed = 0;
		for (int i = 0; i < 100; i++) {
			ResolutionResult result = TestHelper.executeResolution("example01").getResult();
			if (result == ResolutionResult.BOX) {
				successful++;
			} else {
				failed++;
				System.err.println("One failed...");
			}
		}
		System.out.println("Successful runs: " + successful);
		System.out.println("Failed runs: " + failed);
		assertTrue(successful == 10000 && failed == 0);
	}

	@Test
	public void tweetyTest() {
		ResolutionResult result = TestHelper.executeResolution("tweetyTest").getResult();
		System.out.println();
		System.out.println();
		assertEquals("TweetyTest failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example01Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example01").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example01 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example02Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example02").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example02 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example03Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example03").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example03 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example04Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example04").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example04 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example05Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example05").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example05 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example06Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example06").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example06 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example07Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example07").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example07 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example08Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example08").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example08 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void lics_example09Test() {
		ResolutionResult result = TestHelper.executeResolution("lics_example09").getResult();
		System.out.println();
		System.out.println();
		assertEquals("LICS Example09 failed!", ResolutionResult.BOX, result);
	}

	@Test
	public void tptp_agathaTest() {
		Settings.setTemporaryProperty(Settings.SETTING_NEGATE_INPUT, false);
		Settings.setTemporaryProperty(Settings.SETTING_NO_RENAME, true);
		try {
			Scanner scanner = new Scanner(System.in);
			scanner.nextLine();
			ResolutionOutput result = TestHelper.executeResolution("agatha.tptp");
			System.out.println();
			System.out.println();
			assertEquals("TPTP Agatha test failed!", ResolutionResult.BOX, result.getResult());
		} finally {
			Settings.setTemporaryProperty(Settings.SETTING_NEGATE_INPUT, true);
			Settings.setTemporaryProperty(Settings.SETTING_NO_RENAME, false);
		}
	}
}
