package unification;

import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.Test;

import util.TestHelper;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.foresolution.parser.FirstOrderDSLParser;
import at.cl.atp.foresolution.transformations.substitution.Substitution;
import at.cl.atp.foresolution.transformations.substitution.mgu.Unification;
import at.cl.atp.foresolution.util.FormulaFormatter;
import at.cl.atp.foresolution.util.FormulaHelper;

public class UnificationTest {

	@Test
	public void runTest() {
		Formula left = FirstOrderDSLParser.parseAndConvertToFormula(new File(
				"testResources/unificationTest1_left.fores"));
		Formula right = FirstOrderDSLParser.parseAndConvertToFormula(new File(
				"testResources/unificationTest1_right.fores"));
		Substitution mgu = Unification.createMGU(left, right);
		assertTrue("Unification failed", mgu != null);
		System.out.println(mgu.toString());
		Formula newLeft = mgu.applyTo(left);
		Formula newRight = mgu.applyTo(right);
		String leftString = FormulaFormatter.format(newLeft);
		String rightString = FormulaFormatter.format(newRight);
		System.out.println(leftString);
		System.out.println(rightString);
		assertTrue("MGU sub. produced wrong result!", leftString.contentEquals(rightString));
	}

	@Test
	public void matchTestNegative() {
		Formula left = TestHelper.load("matchTest1_left");
		Formula right = TestHelper.load("matchTest1_right");
		Substitution sigma = Unification.createSubstitution(left, right);
		System.out.println(sigma);
		assertTrue(sigma == null);
	}

	@Test
	public void matchTestPositive() {
		Formula left = TestHelper.load("matchTest2_left");
		Formula right = TestHelper.load("matchTest2_right");
		Substitution sigma = Unification.createSubstitution(left, right);
		System.out.println(sigma);
		assertTrue(sigma != null);
	}

	@Test
	public void elementaryMGUpropertiesTest1() {
		Formula left = TestHelper.load("unificationTest2_left");
		Formula right = TestHelper.load("unificationTest2_right");
		System.out.println("ORIGINAL FORMULA LEFT:");
		System.out.println("\t" + FormulaFormatter.format(left));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(left).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(left).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(left).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(left).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(left).toString());
		System.out.println();
		System.out.println("ORIGINAL FORMULA RIGHT:");
		System.out.println("\t" + FormulaFormatter.format(right));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(right).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(right).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(right).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(right).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(right).toString());
		Substitution mgu = Unification.createMGU(left, right);
		System.out.println(mgu);
		assertTrue(mgu != null);
	}

	@Test
	public void elementaryMGUpropertiesTest2() {
		Formula left = TestHelper.load("unificationTest3_left");
		Formula right = TestHelper.load("unificationTest3_right");
		System.out.println("ORIGINAL FORMULA LEFT:");
		System.out.println("\t" + FormulaFormatter.format(left));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(left).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(left).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(left).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(left).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(left).toString());
		System.out.println();
		System.out.println("ORIGINAL FORMULA RIGHT:");
		System.out.println("\t" + FormulaFormatter.format(right));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(right).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(right).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(right).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(right).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(right).toString());
		Substitution mgu = Unification.createMGU(left, right);
		System.out.println(mgu);
		assertTrue(mgu == null);
	}

	@Test
	public void elementaryMGUpropertiesTest3() {
		Formula left = TestHelper.load("unificationTest4_left");
		Formula right = TestHelper.load("unificationTest4_right");
		Substitution mgu = Unification.createMGU(left, right);
		System.out.println("ORIGINAL FORMULA LEFT:");
		System.out.println("\t" + FormulaFormatter.format(left));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(left).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(left).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(left).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(left).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(left).toString());
		System.out.println();
		System.out.println("ORIGINAL FORMULA RIGHT:");
		System.out.println("\t" + FormulaFormatter.format(right));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(right).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(right).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(right).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(right).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(right).toString());
		System.out.println(mgu);
		assertTrue(mgu != null);
		assertTrue(mgu.isEmpty());
	}

	@Test
	public void elementaryMGUpropertiesTest4() {
		Formula left = TestHelper.load("unificationTest5_left");
		Formula right = TestHelper.load("unificationTest5_right");
		System.out.println("ORIGINAL FORMULA LEFT:");
		System.out.println("\t" + FormulaFormatter.format(left));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(left).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(left).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(left).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(left).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(left).toString());
		System.out.println();
		System.out.println("ORIGINAL FORMULA RIGHT:");
		System.out.println("\t" + FormulaFormatter.format(right));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(right).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(right).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(right).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(right).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(right).toString());
		Substitution mgu = Unification.createMGU(left, right);
		System.out.println(mgu);
		assertTrue(mgu != null);
		assertTrue(!mgu.isEmpty());
	}

	@Test
	public void elementaryMGUpropertiesTest5() {
		Formula left = TestHelper.load("unificationTest6_left");
		Formula right = TestHelper.load("unificationTest6_right");
		System.out.println("ORIGINAL FORMULA LEFT:");
		System.out.println("\t" + FormulaFormatter.format(left));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(left).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(left).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(left).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(left).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(left).toString());
		System.out.println();
		System.out.println("ORIGINAL FORMULA RIGHT:");
		System.out.println("\t" + FormulaFormatter.format(right));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(right).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(right).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(right).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(right).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(right).toString());
		Substitution mgu = Unification.createMGU(left, right);
		System.out.println(mgu);
		assertTrue(mgu == null);
	}

	@Test
	public void elementaryMGUpropertiesTest7() {
		Formula left = TestHelper.load("unificationTest7_left");
		Formula right = TestHelper.load("unificationTest7_right");
		System.out.println("ORIGINAL FORMULA LEFT:");
		System.out.println("\t" + FormulaFormatter.format(left));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(left).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(left).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(left).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(left).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(left).toString());
		System.out.println();
		System.out.println("ORIGINAL FORMULA RIGHT:");
		System.out.println("\t" + FormulaFormatter.format(right));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(right).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(right).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(right).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(right).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(right).toString());
		Substitution mgu = Unification.createMGU(left, right);
		System.out.println(mgu);
		assertTrue(mgu != null);
	}
}
