package util;

import java.io.File;

import at.cl.atp.fOResolution.FOResolutionFactory;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.foresolution.model.clause.ClausalForm;
import at.cl.atp.foresolution.model.clause.ClausalFormBuilder;
import at.cl.atp.foresolution.model.container.FormulaContainer;
import at.cl.atp.foresolution.parser.FirstOrderDSLParser;
import at.cl.atp.foresolution.prover.FirstOrderResolutionProver;
import at.cl.atp.foresolution.prover.ResolutionOutput;
import at.cl.atp.foresolution.settings.Settings;
import at.cl.atp.foresolution.transformations.NormalFormTransformations;
import at.cl.atp.foresolution.util.FormulaFormatter;
import at.cl.atp.foresolution.util.FormulaHelper;

public class TestHelper {

	public static ResolutionOutput executeResolution(final String filename) {
		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println("___RUNNING_TEST_" + filename + "__________________");
		long timeBefore = System.currentTimeMillis();
		String trueFilename = filename;
		if (filename.endsWith("tptp") == false && filename.endsWith(".fores") == false) {
			trueFilename = filename + ".fores";
		}
		File sourceFile = new File("testResources/" + trueFilename);
		Formula formula = FirstOrderDSLParser.parseAndConvertToFormula(sourceFile);
		long timeAfterParsing = System.currentTimeMillis();
		FormulaContainer container = new FormulaContainer(formula);
		System.out.println("ORIGINAL FORMULA (from " + sourceFile.getPath() + "):");
		System.out.println("\t" + FormulaFormatter.format(formula));
		System.out.println("\tFree Variables:  " + FormulaHelper.getFreeVariables(formula).toString());
		System.out.println("\tBound Variables: " + FormulaHelper.getBoundVariables(formula).toString());
		System.out.println("\tConstant names:  " + FormulaHelper.getConstantSymbols(formula).toString());
		System.out.println("\tFunction names:  " + FormulaHelper.getFunctionSymbols(formula).toString());
		System.out.println("\tPredicate names: " + FormulaHelper.getPredicateSymbols(formula).toString());
		System.out.println();
		Formula f = null;
		if (Settings.negateInput()) {
			NegatedFormula negation = FOResolutionFactory.eINSTANCE.createNegatedFormula();
			negation.setInnerFormula(formula);
			f = negation;
		} else {
			f = formula;
		}
		container.setFormula(f);
		NormalFormTransformations.toNNF(container);
		System.out.println("Negated and transformed to NNF:");
		System.out.println("\t" + container.toString());
		System.out.println("Rectified Refutational-Skolemized CNF:");
		NormalFormTransformations.toClausalForm(container);
		System.out.println("\t" + container.toString());
		ClausalForm clausalForm = ClausalFormBuilder.build(container);
		System.out.println("Clausal Form:");
		System.out.println("\t" + clausalForm.toString());
		ResolutionOutput result = FirstOrderResolutionProver.run(clausalForm);
		long timeAfter = System.currentTimeMillis();
		System.out.println();
		System.out.println("___RESULT________________________________________");
		System.out.println("Result of resolution: " + result);
		System.out.println();
		System.out.println("___TIME__________________________________________");
		System.out.println("Parse time: " + (timeAfterParsing - timeBefore) + "ms.");
		System.out.println("Resolution: " + (timeAfter - timeAfterParsing) + "ms.");
		System.out.println("Total time: " + (timeAfter - timeBefore) + "ms.");
		long memTotal = Runtime.getRuntime().totalMemory();
		long memMax = Runtime.getRuntime().maxMemory();
		long memUsed = memTotal - Runtime.getRuntime().freeMemory();
		System.out.println();
		System.out.println("___MEMORY________________________________________");
		System.out.println("Available: " + toMegabytes(memMax));
		System.out.println("Allocated: " + toMegabytes(memTotal));
		System.out.println("Used:      " + toMegabytes(memUsed));
		System.gc();
		return result;
	}

	public static Formula load(final String filename) {
		File sourceFile = new File("testResources/" + filename + ".fores");
		Formula formula = FirstOrderDSLParser.parseAndConvertToFormula(sourceFile);
		return formula;
	}

	public static String toMegabytes(final long bytes) {
		double megabytes = bytes;
		megabytes /= 1024;
		megabytes /= 1024;
		return String.format("%.2f MB", megabytes);
	}
}
