package at.cl.atp.foresolution.latexExporter;

import at.cl.atp.fOResolution.FOResolutionFactory;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.foresolution.latexExporter.LatexFormatter;
import at.cl.atp.foresolution.model.clause.ClausalForm;
import at.cl.atp.foresolution.model.clause.ClausalFormBuilder;
import at.cl.atp.foresolution.model.clause.Clause;
import at.cl.atp.foresolution.model.container.FormulaContainer;
import at.cl.atp.foresolution.prover.ResolutionOutput;
import at.cl.atp.foresolution.transformations.NormalFormTransformations;
import at.cl.atp.foresolution.transformations.substitution.Substitution;
import com.google.common.base.Objects;
import java.io.File;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Functions.Function0;
import org.eclipse.xtext.xbase.lib.IntegerRange;

@SuppressWarnings("all")
public class LatexExporter {
  private final static Map<Integer,Substitution> indexSigmaMap = new Function0<Map<Integer,Substitution>>() {
    public Map<Integer,Substitution> apply() {
      HashMap<Integer,Substitution> _hashMap = new HashMap<Integer,Substitution>();
      return _hashMap;
    }
  }.apply();
  
  private final static Map<Substitution,Integer> sigmaIndexMap = new Function0<Map<Substitution,Integer>>() {
    public Map<Substitution,Integer> apply() {
      HashMap<Substitution,Integer> _hashMap = new HashMap<Substitution,Integer>();
      return _hashMap;
    }
  }.apply();
  
  private static int mguIndex = 0;
  
  public static void exportToLatex(final ResolutionOutput output, final Formula formula) {
    try {
      LatexExporter.sigmaIndexMap.clear();
      LatexExporter.indexSigmaMap.clear();
      LatexExporter.mguIndex = 1;
      Substitution _mgu = output.getMgu();
      LatexExporter.sigmaIndexMap.put(_mgu, Integer.valueOf(0));
      Substitution _mgu_1 = output.getMgu();
      LatexExporter.indexSigmaMap.put(Integer.valueOf(0), _mgu_1);
      Clause _leftParentOfBox = output.getLeftParentOfBox();
      LatexExporter.initializeSigmaIndexMap(_leftParentOfBox);
      Clause _rightParentOfBox = output.getRightParentOfBox();
      LatexExporter.initializeSigmaIndexMap(_rightParentOfBox);
      File _file = new File("output.tex");
      final File texFile = _file;
      boolean _exists = texFile.exists();
      boolean _equals = (_exists == false);
      if (_equals) {
        texFile.createNewFile();
      }
      PrintWriter _printWriter = new PrintWriter(texFile);
      final PrintWriter writer = _printWriter;
      String contents = LatexExporter.generateContents(output, formula);
      writer.write(contents);
      writer.flush();
      writer.close();
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  private static void initializeSigmaIndexMap(final Clause clause) {
    Substitution _parentMGU = clause.getParentMGU();
    boolean _notEquals = (!Objects.equal(_parentMGU, null));
    if (_notEquals) {
      Substitution _parentMGU_1 = clause.getParentMGU();
      boolean _containsKey = LatexExporter.sigmaIndexMap.containsKey(_parentMGU_1);
      boolean _equals = (_containsKey == false);
      if (_equals) {
        Substitution _parentMGU_2 = clause.getParentMGU();
        LatexExporter.indexSigmaMap.put(Integer.valueOf(LatexExporter.mguIndex), _parentMGU_2);
        Substitution _parentMGU_3 = clause.getParentMGU();
        LatexExporter.sigmaIndexMap.put(_parentMGU_3, Integer.valueOf(LatexExporter.mguIndex));
        int _plus = (LatexExporter.mguIndex + 1);
        LatexExporter.mguIndex = _plus;
      }
    }
    Clause _leftParent = clause.getLeftParent();
    boolean _notEquals_1 = (!Objects.equal(_leftParent, null));
    if (_notEquals_1) {
      Clause _leftParent_1 = clause.getLeftParent();
      LatexExporter.initializeSigmaIndexMap(_leftParent_1);
    }
    Clause _rightParent = clause.getRightParent();
    boolean _notEquals_2 = (!Objects.equal(_rightParent, null));
    if (_notEquals_2) {
      Clause _rightParent_1 = clause.getRightParent();
      LatexExporter.initializeSigmaIndexMap(_rightParent_1);
    }
  }
  
  private static String generateContents(final ResolutionOutput output, final Formula formula) {
    final NegatedFormula negation = FOResolutionFactory.eINSTANCE.createNegatedFormula();
    Formula _copy = EcoreUtil.<Formula>copy(formula);
    negation.setInnerFormula(_copy);
    FormulaContainer _formulaContainer = new FormulaContainer(negation);
    final FormulaContainer container = _formulaContainer;
    NormalFormTransformations.rectify(container);
    NormalFormTransformations.toNNF(container);
    NormalFormTransformations.structuralSkolemization(container);
    NormalFormTransformations.toClausalForm(container);
    final ClausalForm clausalForm = ClausalFormBuilder.build(container);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("\\documentclass[border=1cm,varwidth=\\maxdimen]{standalone}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\\usepackage{bussproofs}");
    _builder.newLine();
    _builder.append("\\usepackage{amssymb}");
    _builder.newLine();
    _builder.newLine();
    _builder.newLine();
    _builder.append("\\begin{document}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\\textbf{The formula}\\\\");
    _builder.newLine();
    _builder.append("This is the proof of the following formula:\\\\");
    _builder.newLine();
    _builder.append("\\begin{center}");
    _builder.newLine();
    String _format = LatexFormatter.format(formula);
    _builder.append(_format, "");
    _builder.newLineIfNotEmpty();
    _builder.append("\\end{center}");
    _builder.newLine();
    _builder.append("\\vspace{1cm}");
    _builder.newLine();
    _builder.append("\\textbf{The NNF and Clausal Forms}\\\\");
    _builder.newLine();
    _builder.append("The skolemized Negation Normal Form of the negated input:");
    _builder.newLine();
    _builder.append("\\begin{center}");
    _builder.newLine();
    Formula _formula = container.getFormula();
    String _format_1 = LatexFormatter.format(_formula);
    _builder.append(_format_1, "");
    _builder.newLineIfNotEmpty();
    _builder.append("\\end{center}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\\noindent The corresponding clausal form:");
    _builder.newLine();
    _builder.append("\\begin{center}");
    _builder.newLine();
    String _format_2 = LatexFormatter.format(clausalForm);
    _builder.append(_format_2, "");
    _builder.newLineIfNotEmpty();
    _builder.append("\\end{center}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\\vspace{1cm}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\\textbf{The proof}");
    _builder.newLine();
    _builder.append("\\begin{prooftree}");
    _builder.newLine();
    Clause _leftParentOfBox = output.getLeftParentOfBox();
    String _generateProof = LatexExporter.generateProof(_leftParentOfBox);
    _builder.append(_generateProof, "");
    _builder.newLineIfNotEmpty();
    Clause _rightParentOfBox = output.getRightParentOfBox();
    String _generateProof_1 = LatexExporter.generateProof(_rightParentOfBox);
    _builder.append(_generateProof_1, "");
    _builder.newLineIfNotEmpty();
    _builder.append("\\RightLabel{$ \\sigma _");
    Substitution _mgu = output.getMgu();
    Integer _get = LatexExporter.sigmaIndexMap.get(_mgu);
    _builder.append(_get, "");
    _builder.append(" $}");
    _builder.newLineIfNotEmpty();
    _builder.append("\\BinaryInfC{$\\Box$}");
    _builder.newLine();
    _builder.append("\\end{prooftree}");
    _builder.newLine();
    _builder.append("\\vspace{1cm}");
    _builder.newLine();
    _builder.append("\\textbf{The substitutions}\\\\");
    _builder.newLine();
    {
      int _minus = (LatexExporter.mguIndex - 1);
      IntegerRange _upTo = new IntegerRange(0, _minus);
      for(final Integer i : _upTo) {
        _builder.append("$ \\sigma _{");
        _builder.append(i, "");
        _builder.append(" = ");
        Substitution _get_1 = LatexExporter.indexSigmaMap.get(i);
        String _format_3 = LatexFormatter.format(_get_1);
        _builder.append(_format_3, "");
        _builder.append("}$\\\\");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    _builder.append("\\end{document}");
    _builder.newLine();
    return _builder.toString();
  }
  
  private static String generateProof(final Clause clause) {
    String _string = clause.toString();
    String _plus = ("Generating proof for: " + _string);
    System.out.print(_plus);
    boolean _and = false;
    Clause _leftParent = clause.getLeftParent();
    boolean _equals = Objects.equal(_leftParent, null);
    if (!_equals) {
      _and = false;
    } else {
      Clause _rightParent = clause.getRightParent();
      boolean _equals_1 = Objects.equal(_rightParent, null);
      _and = (_equals && _equals_1);
    }
    if (_and) {
      System.out.println(" which is a leaf.");
      System.out.print("FORMAT AXIOM: ");
      final String output = LatexFormatter.format(clause);
      String _plus_1 = ("\t\tLaTeX rep: " + output);
      System.out.println(_plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("\\AxiomC{}");
      _builder.newLine();
      _builder.append("\\RightLabel{ax}");
      _builder.newLine();
      _builder.append("\\UnaryInfC{ ");
      _builder.append(output, "");
      _builder.append(" }");
      _builder.newLineIfNotEmpty();
      return _builder.toString();
    } else {
      boolean _and_1 = false;
      Clause _leftParent_1 = clause.getLeftParent();
      boolean _notEquals = (!Objects.equal(_leftParent_1, null));
      if (!_notEquals) {
        _and_1 = false;
      } else {
        Clause _rightParent_1 = clause.getRightParent();
        boolean _equals_2 = Objects.equal(_rightParent_1, null);
        _and_1 = (_notEquals && _equals_2);
      }
      if (_and_1) {
        StringConcatenation _builder_1 = new StringConcatenation();
        Clause _leftParent_2 = clause.getLeftParent();
        String _generateProof = LatexExporter.generateProof(_leftParent_2);
        _builder_1.append(_generateProof, "");
        _builder_1.newLineIfNotEmpty();
        _builder_1.append("\\RightLabel{ fac }");
        _builder_1.newLine();
        _builder_1.append("\\UnaryInfC{");
        String _format = LatexFormatter.format(clause);
        _builder_1.append(_format, "");
        _builder_1.append("}");
        _builder_1.newLineIfNotEmpty();
        return _builder_1.toString();
      } else {
        Clause _leftParent_3 = clause.getLeftParent();
        String _string_1 = _leftParent_3.toString();
        String _plus_2 = (" with parents: " + _string_1);
        String _plus_3 = (_plus_2 + ", ");
        Clause _rightParent_2 = clause.getRightParent();
        String _string_2 = _rightParent_2.toString();
        String _plus_4 = (_plus_3 + _string_2);
        System.out.println(_plus_4);
        StringConcatenation _builder_2 = new StringConcatenation();
        Clause _leftParent_4 = clause.getLeftParent();
        String _generateProof_1 = LatexExporter.generateProof(_leftParent_4);
        _builder_2.append(_generateProof_1, "");
        _builder_2.newLineIfNotEmpty();
        Clause _rightParent_3 = clause.getRightParent();
        String _generateProof_2 = LatexExporter.generateProof(_rightParent_3);
        _builder_2.append(_generateProof_2, "");
        _builder_2.newLineIfNotEmpty();
        _builder_2.append("\\RightLabel{ res $ \\sigma _{");
        Substitution _parentMGU = clause.getParentMGU();
        Integer _get = LatexExporter.sigmaIndexMap.get(_parentMGU);
        _builder_2.append(_get, "");
        _builder_2.append("} $}");
        _builder_2.newLineIfNotEmpty();
        _builder_2.append("\\BinaryInfC{");
        String _format_1 = LatexFormatter.format(clause);
        _builder_2.append(_format_1, "");
        _builder_2.append("}");
        _builder_2.newLineIfNotEmpty();
        return _builder_2.toString();
      }
    }
  }
}
