package at.cl.atp.foresolution.latexExporter;

import at.cl.atp.fOResolution.Atom;
import at.cl.atp.fOResolution.BinaryOperation;
import at.cl.atp.fOResolution.Formula;
import at.cl.atp.fOResolution.Function;
import at.cl.atp.fOResolution.LowBindFormula;
import at.cl.atp.fOResolution.NegatedFormula;
import at.cl.atp.fOResolution.Predicate;
import at.cl.atp.fOResolution.QuantifiedFormula;
import at.cl.atp.fOResolution.Quantifier;
import at.cl.atp.foresolution.alphabet.GreekAlphabet;
import at.cl.atp.foresolution.model.clause.ClausalForm;
import at.cl.atp.foresolution.model.clause.Clause;
import at.cl.atp.foresolution.model.clause.ClauseLiteral;
import at.cl.atp.foresolution.transformations.substitution.Substitution;
import at.cl.atp.foresolution.util.FormulaFormatter;
import com.google.common.base.Objects;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class LatexFormatter {
  public static String format(final Clause clause) {
    final String clauseToString = clause.toString();
    String _plus = ("format clause: " + clauseToString);
    System.out.println(_plus);
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder builder = _stringBuilder;
    builder.append("\\{ $");
    Set<ClauseLiteral> _literals = clause.getLiterals();
    Iterator<ClauseLiteral> literalIterator = _literals.iterator();
    boolean _hasNext = literalIterator.hasNext();
    boolean _while = _hasNext;
    while (_while) {
      {
        ClauseLiteral _next = literalIterator.next();
        String _format = LatexFormatter.format(_next);
        builder.append(_format);
        boolean _or = false;
        boolean _hasNext_1 = literalIterator.hasNext();
        if (_hasNext_1) {
          _or = true;
        } else {
          Set<ClauseLiteral> _negatedLiterals = clause.getNegatedLiterals();
          boolean _isEmpty = _negatedLiterals.isEmpty();
          boolean _equals = (_isEmpty == false);
          _or = (_hasNext_1 || _equals);
        }
        if (_or) {
          builder.append(", ");
        }
      }
      boolean _hasNext_1 = literalIterator.hasNext();
      _while = _hasNext_1;
    }
    Set<ClauseLiteral> _negatedLiterals = clause.getNegatedLiterals();
    Iterator<ClauseLiteral> _iterator = _negatedLiterals.iterator();
    literalIterator = _iterator;
    boolean _hasNext_1 = literalIterator.hasNext();
    boolean _while_1 = _hasNext_1;
    while (_while_1) {
      {
        ClauseLiteral _next = literalIterator.next();
        String _format = LatexFormatter.format(_next);
        builder.append(_format);
        boolean _hasNext_2 = literalIterator.hasNext();
        if (_hasNext_2) {
          builder.append(", ");
        }
      }
      boolean _hasNext_2 = literalIterator.hasNext();
      _while_1 = _hasNext_2;
    }
    builder.append(" $ \\}");
    return builder.toString();
  }
  
  public static String format(final ClauseLiteral literal) {
    String _string = literal.toString();
    String _plus = ("format literal: " + _string);
    System.out.println(_plus);
    String _string_1 = literal.toString();
    boolean _contentEquals = _string_1.contentEquals("A(beta, b(beta))");
    if (_contentEquals) {
      System.out.println("CRIT");
    }
    Formula _formula = literal.getFormula();
    return LatexFormatter.formatFormula(_formula);
  }
  
  public static String format(final Formula formula) {
    String _format = FormulaFormatter.format(formula);
    String _plus = ("format formula: " + _format);
    System.out.println(_plus);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("$ ");
    String _formatFormula = LatexFormatter.formatFormula(formula);
    _builder.append(_formatFormula, "");
    _builder.append(" $");
    return _builder.toString();
  }
  
  public static String format(final Substitution sigma) {
    String _string = sigma.toString();
    String _plus = ("Formatting substitution: " + _string);
    System.out.println(_plus);
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder builder = _stringBuilder;
    builder.append("\\{ ");
    Map<String,Formula> _replacements = sigma.getReplacements();
    Set<Entry<String,Formula>> _entrySet = _replacements.entrySet();
    final Iterator<Entry<String,Formula>> iterator = _entrySet.iterator();
    boolean _hasNext = iterator.hasNext();
    boolean _while = _hasNext;
    while (_while) {
      {
        final Entry<String,Formula> current = iterator.next();
        String _key = current.getKey();
        String _formatIdentifier = LatexFormatter.formatIdentifier(_key);
        builder.append(_formatIdentifier);
        builder.append(" \\mapsto ");
        Formula _value = current.getValue();
        String _formatFormula = LatexFormatter.formatFormula(_value);
        builder.append(_formatFormula);
        boolean _hasNext_1 = iterator.hasNext();
        if (_hasNext_1) {
          builder.append(", ");
        }
      }
      boolean _hasNext_1 = iterator.hasNext();
      _while = _hasNext_1;
    }
    builder.append(" \\}");
    return builder.toString();
  }
  
  public static String format(final ClausalForm clausalForm) {
    final Iterator<Clause> iterator = clausalForm.clauseIterator();
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder builder = _stringBuilder;
    boolean _hasNext = iterator.hasNext();
    boolean _while = _hasNext;
    while (_while) {
      {
        final Clause current = iterator.next();
        String _format = LatexFormatter.format(current);
        builder.append(_format);
        boolean _hasNext_1 = iterator.hasNext();
        if (_hasNext_1) {
          builder.append(", ");
        }
      }
      boolean _hasNext_1 = iterator.hasNext();
      _while = _hasNext_1;
    }
    return builder.toString();
  }
  
  protected static String _formatFormula(final Atom atom) {
    String _format = FormulaFormatter.format(atom);
    String _plus = ("formatFormula atom: " + _format);
    System.out.println(_plus);
    String _name = atom.getName();
    return LatexFormatter.formatIdentifier(_name);
  }
  
  protected static String _formatFormula(final NegatedFormula negatedFormula) {
    String _format = FormulaFormatter.format(negatedFormula);
    String _plus = ("formatFormula negated: " + _format);
    System.out.println(_plus);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("\\neg ");
    Formula _innerFormula = negatedFormula.getInnerFormula();
    String _formatFormula = LatexFormatter.formatFormula(_innerFormula);
    _builder.append(_formatFormula, "");
    return _builder.toString();
  }
  
  protected static String _formatFormula(final Predicate predicate) {
    final String predicateToString = FormulaFormatter.format(predicate);
    String _plus = ("formatFormula predicate: " + predicateToString);
    System.out.println(_plus);
    boolean _contentEquals = predicateToString.contentEquals("A(beta, b(beta))");
    if (_contentEquals) {
      System.out.println("CRIT");
    }
    EList<Formula> _arguments = predicate.getArguments();
    final Iterator<Formula> iterator = _arguments.iterator();
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder builder = _stringBuilder;
    String _name = predicate.getName();
    String _formatIdentifier = LatexFormatter.formatIdentifier(_name);
    builder.append(_formatIdentifier);
    builder.append("(");
    boolean _hasNext = iterator.hasNext();
    boolean _while = _hasNext;
    while (_while) {
      {
        Formula _next = iterator.next();
        String _formatFormula = LatexFormatter.formatFormula(_next);
        builder.append(_formatFormula);
        boolean _hasNext_1 = iterator.hasNext();
        if (_hasNext_1) {
          builder.append(", ");
        }
      }
      boolean _hasNext_1 = iterator.hasNext();
      _while = _hasNext_1;
    }
    builder.append(")");
    return builder.toString();
  }
  
  protected static String _formatFormula(final Function function) {
    String _format = FormulaFormatter.format(function);
    String _plus = ("formatFormula function: " + _format);
    System.out.println(_plus);
    EList<Formula> _arguments = function.getArguments();
    final Iterator<Formula> iterator = _arguments.iterator();
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder builder = _stringBuilder;
    String _name = function.getName();
    String _formatIdentifier = LatexFormatter.formatIdentifier(_name);
    builder.append(_formatIdentifier);
    builder.append("(");
    boolean _hasNext = iterator.hasNext();
    boolean _while = _hasNext;
    while (_while) {
      {
        Formula _next = iterator.next();
        String _formatFormula = LatexFormatter.formatFormula(_next);
        builder.append(_formatFormula);
        boolean _hasNext_1 = iterator.hasNext();
        if (_hasNext_1) {
          builder.append(", ");
        }
      }
      boolean _hasNext_1 = iterator.hasNext();
      _while = _hasNext_1;
    }
    builder.append(")");
    return builder.toString();
  }
  
  protected static String _formatFormula(final LowBindFormula formula) {
    String _format = FormulaFormatter.format(formula);
    String _plus = ("formatFormula lowbind: " + _format);
    System.out.println(_plus);
    StringConcatenation _builder = new StringConcatenation();
    Formula _left = formula.getLeft();
    String _formatFormula = LatexFormatter.formatFormula(_left);
    _builder.append(_formatFormula, "");
    _builder.append(" ");
    BinaryOperation _operation = formula.getOperation();
    String _operationSymbol = LatexFormatter.operationSymbol(_operation);
    _builder.append(_operationSymbol, "");
    _builder.append(" ");
    Formula _right = formula.getRight();
    String _formatFormula_1 = LatexFormatter.formatFormula(_right);
    _builder.append(_formatFormula_1, "");
    return _builder.toString();
  }
  
  protected static String _formatFormula(final QuantifiedFormula formula) {
    String _format = FormulaFormatter.format(formula);
    String _plus = ("formatFormula quantified: " + _format);
    System.out.println(_plus);
    EList<String> _variables = formula.getVariables();
    final Iterator<String> iterator = _variables.iterator();
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder builder = _stringBuilder;
    Quantifier _quantifier = formula.getQuantifier();
    String _quantorSymbol = LatexFormatter.quantorSymbol(_quantifier);
    builder.append(_quantorSymbol);
    builder.append(" ");
    boolean _hasNext = iterator.hasNext();
    boolean _while = _hasNext;
    while (_while) {
      {
        String _next = iterator.next();
        String _formatIdentifier = LatexFormatter.formatIdentifier(_next);
        builder.append(_formatIdentifier);
        boolean _hasNext_1 = iterator.hasNext();
        if (_hasNext_1) {
          builder.append(", ");
        }
      }
      boolean _hasNext_1 = iterator.hasNext();
      _while = _hasNext_1;
    }
    builder.append("(");
    Formula _innerFormula = formula.getInnerFormula();
    String _formatFormula = LatexFormatter.formatFormula(_innerFormula);
    builder.append(_formatFormula);
    builder.append(")");
    return builder.toString();
  }
  
  public static String operationSymbol(final BinaryOperation operation) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(operation,BinaryOperation.CONJUNCTION)) {
        _matched=true;
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("\\land");
        return _builder.toString();
      }
    }
    if (!_matched) {
      if (Objects.equal(operation,BinaryOperation.DISJUNCTION)) {
        _matched=true;
        StringConcatenation _builder_1 = new StringConcatenation();
        _builder_1.append("\\lor");
        return _builder_1.toString();
      }
    }
    if (!_matched) {
      if (Objects.equal(operation,BinaryOperation.IMPLICATION)) {
        _matched=true;
        StringConcatenation _builder_2 = new StringConcatenation();
        _builder_2.append("\\rightarrow");
        return _builder_2.toString();
      }
    }
    return _switchResult;
  }
  
  public static String quantorSymbol(final Quantifier quantifier) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(quantifier,Quantifier.EXISTENTIAL)) {
        _matched=true;
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("\\exists");
        return _builder.toString();
      }
    }
    if (!_matched) {
      if (Objects.equal(quantifier,Quantifier.UNIVERSAL)) {
        _matched=true;
        StringConcatenation _builder_1 = new StringConcatenation();
        _builder_1.append("\\forall");
        return _builder_1.toString();
      }
    }
    return _switchResult;
  }
  
  public static String formatIdentifier(final String identifier) {
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder builder = _stringBuilder;
    final Pattern pattern = Pattern.compile("[0-9]+");
    final Matcher matcher = pattern.matcher(identifier);
    int index = (-1);
    String ident = identifier.replaceAll("\\_", "");
    boolean _find = matcher.find();
    if (_find) {
      String _group = matcher.group();
      int _parseInt = Integer.parseInt(_group);
      index = _parseInt;
      String _replaceAll = ident.replaceAll("[0-9]", "");
      ident = _replaceAll;
    }
    char[] _charArray = ident.toCharArray();
    final Procedure1<Character> _function = new Procedure1<Character>() {
        public void apply(final Character character) {
          boolean _contains = GreekAlphabet.instance.contains((character).charValue());
          if (_contains) {
            builder.append("\\");
            String _latex = GreekAlphabet.instance.toLatex((character).charValue());
            builder.append(_latex);
            builder.append(" ");
          } else {
            builder.append(character);
          }
        }
      };
    IterableExtensions.<Character>forEach(((Iterable<Character>)Conversions.doWrapArray(_charArray)), _function);
    boolean _greaterThan = (index > 0);
    if (_greaterThan) {
      builder.append("_");
      builder.append(index);
    }
    return builder.toString();
  }
  
  public static String formatFormula(final Formula atom) {
    if (atom instanceof Atom) {
      return _formatFormula((Atom)atom);
    } else if (atom instanceof Function) {
      return _formatFormula((Function)atom);
    } else if (atom instanceof LowBindFormula) {
      return _formatFormula((LowBindFormula)atom);
    } else if (atom instanceof NegatedFormula) {
      return _formatFormula((NegatedFormula)atom);
    } else if (atom instanceof Predicate) {
      return _formatFormula((Predicate)atom);
    } else if (atom instanceof QuantifiedFormula) {
      return _formatFormula((QuantifiedFormula)atom);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(atom).toString());
    }
  }
}
