package at.cl.atp.foresolution.latexExporter

import at.cl.atp.foresolution.model.clause.Clause
import at.cl.atp.foresolution.model.clause.ClauseLiteral
import at.cl.atp.fOResolution.NegatedFormula
import at.cl.atp.fOResolution.Predicate
import at.cl.atp.fOResolution.Function
import at.cl.atp.fOResolution.Atom
import java.util.regex.Pattern
import java.util.regex.Matcher
import at.cl.atp.foresolution.alphabet.GreekAlphabet
import at.cl.atp.fOResolution.LowBindFormula
import at.cl.atp.fOResolution.BinaryOperation
import at.cl.atp.fOResolution.Quantifier
import at.cl.atp.fOResolution.QuantifiedFormula
import at.cl.atp.fOResolution.Formula
import at.cl.atp.foresolution.util.FormulaFormatter
import java.util.Iterator
import at.cl.atp.foresolution.transformations.substitution.Substitution
import java.util.Map$Entry
import at.cl.atp.foresolution.model.clause.ClausalForm

class LatexFormatter {
	
	
	def static String format(Clause clause){
		val String clauseToString = clause.toString;
		System::out.println("format clause: " + clauseToString);
		val StringBuilder builder = new StringBuilder();
		builder.append("\\{ $");
		var Iterator<ClauseLiteral> literalIterator = clause.literals.iterator;
		while(literalIterator.hasNext){
			builder.append(format(literalIterator.next));
			if(literalIterator.hasNext || clause.negatedLiterals.empty == false){
				builder.append(", ");
			}
		}
		literalIterator = clause.negatedLiterals.iterator;
		while(literalIterator.hasNext){
			builder.append(format(literalIterator.next));
			if(literalIterator.hasNext){
				builder.append(", ");
			}
		}
		builder.append(" $ \\}");
		return builder.toString;
	}
	
	def static String format(ClauseLiteral literal){
		System::out.println("format literal: " + literal.toString);
		if(literal.toString.contentEquals("A(beta, b(beta))")){
			System::out.println("CRIT");
		}
		return formatFormula(literal.formula);
	}
	
	def static String format(Formula formula){
		System::out.println("format formula: " + FormulaFormatter::format(formula));
		return '''$ «formatFormula(formula)» $'''
	}
	
	def static String format(Substitution sigma){
		System::out.println("Formatting substitution: " + sigma.toString);
		val StringBuilder builder = new StringBuilder();
		builder.append("\\{ ");
		val Iterator<Entry<String, Formula>> iterator = sigma.replacements.entrySet.iterator;
		while(iterator.hasNext){
			val Entry<String, Formula> current = iterator.next;
			builder.append(formatIdentifier(current.key));
			builder.append(" \\mapsto ");
			builder.append(formatFormula(current.value));
			if(iterator.hasNext){
				builder.append(", ");
			}
		}
		builder.append(" \\}");
		return builder.toString;
	}
	
	def static String format(ClausalForm clausalForm){
		val Iterator<Clause> iterator = clausalForm.clauseIterator;
		val StringBuilder builder = new StringBuilder;
		while(iterator.hasNext){
			val Clause current = iterator.next;
			builder.append(format(current));
			if(iterator.hasNext){
				builder.append(", ");
			}	
		}
		return builder.toString;
	}
	
	def static dispatch String formatFormula(Atom atom){
		System::out.println("formatFormula atom: " + FormulaFormatter::format(atom));
		return formatIdentifier(atom.name);
	}
	
	def static dispatch String formatFormula(NegatedFormula negatedFormula){
		System::out.println("formatFormula negated: " + FormulaFormatter::format(negatedFormula));
		return '''\neg «formatFormula(negatedFormula.innerFormula)»''';
	}
	
	def static dispatch String formatFormula(Predicate predicate){
		val String predicateToString = FormulaFormatter::format(predicate);
		System::out.println("formatFormula predicate: " + predicateToString);
		if(predicateToString.contentEquals("A(beta, b(beta))")){
			System::out.println("CRIT");
		}
		val Iterator<Formula> iterator = predicate.arguments.iterator;
		val StringBuilder builder = new StringBuilder();
		builder.append(formatIdentifier(predicate.name));
		builder.append("(");
		while(iterator.hasNext){
			builder.append(formatFormula(iterator.next));
			if(iterator.hasNext){
				builder.append(", ");
			}
		}
		builder.append(")");
		return builder.toString;
	}
	
	def static dispatch String formatFormula(Function function){
		System::out.println("formatFormula function: " + FormulaFormatter::format(function));
		val Iterator<Formula> iterator = function.arguments.iterator;
		val StringBuilder builder = new StringBuilder();
		builder.append(formatIdentifier(function.name));
		builder.append("(");
		while(iterator.hasNext){
			builder.append(formatFormula(iterator.next));
			if(iterator.hasNext){
				builder.append(", ");
			}
		}
		builder.append(")");
		return builder.toString;
	}
	
	def static dispatch String formatFormula(LowBindFormula formula){
		System::out.println("formatFormula lowbind: " + FormulaFormatter::format(formula));
		return '''«formatFormula(formula.left)» «operationSymbol(formula.operation)» «formatFormula(formula.right)»'''
	}
	
	def static dispatch String formatFormula(QuantifiedFormula formula){
		System::out.println("formatFormula quantified: " + FormulaFormatter::format(formula));
		val Iterator<String> iterator = formula.variables.iterator;
		val StringBuilder builder = new StringBuilder();
		builder.append(quantorSymbol(formula.quantifier));
		builder.append(" ");
		while(iterator.hasNext){
			builder.append(formatIdentifier(iterator.next));
			if(iterator.hasNext){
				builder.append(", ");
			}
		}
		builder.append("(");
		builder.append(formatFormula(formula.innerFormula));
		builder.append(")");
		return builder.toString;
	}
	
	def static String operationSymbol(BinaryOperation operation){
		switch(operation){
			case BinaryOperation::CONJUNCTION:
				return '''\land'''
			case BinaryOperation::DISJUNCTION:
				return '''\lor'''
			case BinaryOperation::IMPLICATION:
				return '''\rightarrow'''
		}
	}
	
	def static String quantorSymbol(Quantifier quantifier){
		switch(quantifier){
			case Quantifier::EXISTENTIAL:
				return '''\exists'''
			case Quantifier::UNIVERSAL:
				return '''\forall'''
		}
	}
	
	def static String formatIdentifier(String identifier){
		val StringBuilder builder = new StringBuilder();
		val Pattern pattern = Pattern::compile("[0-9]+");
		val Matcher matcher = pattern.matcher(identifier);
		var int index = -1;
		var String ident = identifier.replaceAll("\\_", "");
		if(matcher.find){
			index = Integer::parseInt(matcher.group);
			ident = ident.replaceAll("[0-9]", "");
		}
		ident.toCharArray.forEach[character | 
			if(GreekAlphabet::instance.contains(character)){
				builder.append("\\");
				builder.append(GreekAlphabet::instance.toLatex(character));
				builder.append(" ");
			}else{
				builder.append(character);
			}
		]
		if(index > 0){
			builder.append("_");
			builder.append(index);
		}
		return builder.toString;
	}
}