package at.cl.atp.serializer;

import at.cl.atp.resolution.AndFormula;
import at.cl.atp.resolution.Atom;
import at.cl.atp.resolution.ImplicationFormula;
import at.cl.atp.resolution.NegatedFormula;
import at.cl.atp.resolution.OrFormula;
import at.cl.atp.resolution.ResolutionPackage;
import at.cl.atp.resolution.ResolutionProblem;
import at.cl.atp.services.ResolutionGrammarAccess;
import com.google.inject.Inject;
import com.google.inject.Provider;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractDelegatingSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("all")
public class ResolutionSemanticSequencer extends AbstractDelegatingSemanticSequencer {

	@Inject
	private ResolutionGrammarAccess grammarAccess;
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == ResolutionPackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case ResolutionPackage.AND_FORMULA:
				if(context == grammarAccess.getAndFormulaRule() ||
				   context == grammarAccess.getConjunctiveFormulaRule() ||
				   context == grammarAccess.getFormulaRule()) {
					sequence_AndFormula(context, (AndFormula) semanticObject); 
					return; 
				}
				else break;
			case ResolutionPackage.ATOM:
				if(context == grammarAccess.getAtomRule() ||
				   context == grammarAccess.getFormulaRule()) {
					sequence_Atom(context, (Atom) semanticObject); 
					return; 
				}
				else break;
			case ResolutionPackage.IMPLICATION_FORMULA:
				if(context == grammarAccess.getConjunctiveFormulaRule() ||
				   context == grammarAccess.getFormulaRule() ||
				   context == grammarAccess.getImplicationFormulaRule()) {
					sequence_ImplicationFormula(context, (ImplicationFormula) semanticObject); 
					return; 
				}
				else break;
			case ResolutionPackage.NEGATED_FORMULA:
				if(context == grammarAccess.getFormulaRule() ||
				   context == grammarAccess.getNegatedFormulaRule()) {
					sequence_NegatedFormula(context, (NegatedFormula) semanticObject); 
					return; 
				}
				else break;
			case ResolutionPackage.OR_FORMULA:
				if(context == grammarAccess.getConjunctiveFormulaRule() ||
				   context == grammarAccess.getFormulaRule() ||
				   context == grammarAccess.getOrFormulaRule()) {
					sequence_OrFormula(context, (OrFormula) semanticObject); 
					return; 
				}
				else break;
			case ResolutionPackage.RESOLUTION_PROBLEM:
				if(context == grammarAccess.getResolutionProblemRule()) {
					sequence_ResolutionProblem(context, (ResolutionProblem) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     (first=Formula second=Formula)
	 */
	protected void sequence_AndFormula(EObject context, AndFormula semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__FIRST) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__FIRST));
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__SECOND) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__SECOND));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getAndFormulaAccess().getFirstFormulaParserRuleCall_2_0(), semanticObject.getFirst());
		feeder.accept(grammarAccess.getAndFormulaAccess().getSecondFormulaParserRuleCall_4_0(), semanticObject.getSecond());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     name=ID
	 */
	protected void sequence_Atom(EObject context, Atom semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.ATOM__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.ATOM__NAME));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getAtomAccess().getNameIDTerminalRuleCall_0(), semanticObject.getName());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (first=Formula second=Formula)
	 */
	protected void sequence_ImplicationFormula(EObject context, ImplicationFormula semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__FIRST) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__FIRST));
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__SECOND) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__SECOND));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getImplicationFormulaAccess().getFirstFormulaParserRuleCall_2_0(), semanticObject.getFirst());
		feeder.accept(grammarAccess.getImplicationFormulaAccess().getSecondFormulaParserRuleCall_4_0(), semanticObject.getSecond());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     innerFormula=Formula
	 */
	protected void sequence_NegatedFormula(EObject context, NegatedFormula semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.NEGATED_FORMULA__INNER_FORMULA) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.NEGATED_FORMULA__INNER_FORMULA));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getNegatedFormulaAccess().getInnerFormulaFormulaParserRuleCall_2_0(), semanticObject.getInnerFormula());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (first=Formula second=Formula)
	 */
	protected void sequence_OrFormula(EObject context, OrFormula semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__FIRST) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__FIRST));
			if(transientValues.isValueTransient(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__SECOND) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, ResolutionPackage.Literals.CONJUNCTIVE_FORMULA__SECOND));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getOrFormulaAccess().getFirstFormulaParserRuleCall_2_0(), semanticObject.getFirst());
		feeder.accept(grammarAccess.getOrFormulaAccess().getSecondFormulaParserRuleCall_4_0(), semanticObject.getSecond());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (formulas+=Formula formulas+=Formula*)
	 */
	protected void sequence_ResolutionProblem(EObject context, ResolutionProblem semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
}
