package app;

import at.cl.atp.resolution.AndFormula;
import at.cl.atp.resolution.Atom;
import at.cl.atp.resolution.Formula;
import at.cl.atp.resolution.ImplicationFormula;
import at.cl.atp.resolution.NegatedFormula;
import at.cl.atp.resolution.OrFormula;

public class FormulaValidator {

	public static void validateFormulaIntegrity(final Formula formula) {
		if (formula == null) {
			throw new RuntimeException("Formula invalid: NULL");
		} else if (formula instanceof Atom) {
			Atom atom = (Atom) formula;
			if (atom.getName() == null || atom.getName().trim().isEmpty()) {
				throw new RuntimeException("Formula invalid: Atom name unspecified");
			}
		} else if (formula instanceof NegatedFormula) {
			NegatedFormula negation = (NegatedFormula) formula;
			if (negation.getInnerFormula() == null) {
				throw new RuntimeException("Formula invalid: Negation innner formula is NULL");
			} else {
				validateFormulaIntegrity(negation.getInnerFormula());
			}
		} else if (formula instanceof AndFormula) {
			AndFormula conjunction = (AndFormula) formula;
			if (conjunction.getFirst() == null) {
				throw new RuntimeException("Formula invalid: Conjunction first argument is NULL");
			} else if (conjunction.getSecond() == null) {
				throw new RuntimeException("Formula invalid: Conjunction second argument is NULL");
			} else {
				validateFormulaIntegrity(conjunction.getFirst());
				validateFormulaIntegrity(conjunction.getSecond());
			}
		} else if (formula instanceof OrFormula) {
			OrFormula disjunction = (OrFormula) formula;
			if (disjunction.getFirst() == null) {
				throw new RuntimeException("Formula invalid: Disjunction first argument is NULL");
			} else if (disjunction.getSecond() == null) {
				throw new RuntimeException("Formula invalid: Disjunction second argument is NULL");
			} else {
				validateFormulaIntegrity(disjunction.getFirst());
				validateFormulaIntegrity(disjunction.getSecond());
			}
		} else if (formula instanceof ImplicationFormula) {
			ImplicationFormula implication = (ImplicationFormula) formula;
			if (implication.getFirst() == null) {
				throw new RuntimeException("Formula invalid: Disjunction first argument is NULL");
			} else if (implication.getSecond() == null) {
				throw new RuntimeException("Formula invalid: Disjunction second argument is NULL");
			} else {
				validateFormulaIntegrity(implication.getFirst());
				validateFormulaIntegrity(implication.getSecond());
			}
		} else {
			throw new RuntimeException("Formula invalid: Unknown formula type: " + formula.getClass().getName());
		}
	}
}
