package app;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import app.model.ClausalForm;
import app.model.Clause;

public class ResolutionProver {

	public static ResolutionResult run(final ClausalForm clausalForm) {
		tautologyRule(clausalForm);
		if (clausalForm.getClauses().isEmpty()) {
			return ResolutionResult.EMPTY_SET;
		}
		// sort clauses by size, ascending.
		// unit clauses will be eliminated first
		Collections.sort(clausalForm.getClauses(), new ClauseSizeComparator());
		boolean changed = true;
		while (changed) {
			changed = false;
			List<Clause> clauses = new ArrayList<Clause>(clausalForm.getClauses());
			for (Clause first : clauses) {
				for (Clause second : clauses) {
					if (first != second) {
						if (first.hasClashedWith(second) == false) {
							Set<Clause> clashedClauses = clash(first, second);
							first.registerClash(second);
							second.registerClash(first);
							for (Clause clashedClause : clashedClauses) {
								if (clashedClause != null) {
									if (clashedClause.isEmpty()) {
										return ResolutionResult.BOX;
									} else {
										clausalForm.getClauses().add(clashedClause);
										changed = true;
									}
								}
							}
						}
					}
				}
			}
		}
		for (Clause clause : clausalForm.getClauses()) {
			if (clause.isEmpty()) {
				return ResolutionResult.BOX;
			}
		}
		return ResolutionResult.SATISFIABLE;
	}

	public static void tautologyRule(final ClausalForm clausalForm) {
		List<Clause> iterable = new ArrayList<Clause>(clausalForm.getClauses());
		for (Clause clause : iterable) {
			if (clause.isTautology()) {
				clausalForm.getClauses().remove(clause);
			}
		}
	}

	public static Set<Clause> clash(final Clause first, final Clause second) {
		Set<Clause> clashedClauses = new HashSet<Clause>();
		Set<String> atoms = getClashingAtoms(first, second);
		if (atoms.isEmpty()) {
			return Collections.emptySet();
		}
		for (String atom : atoms) {
			Clause newClause = new Clause();
			newClause.getAtoms().addAll(first.getAtoms());
			newClause.getAtoms().addAll(second.getAtoms());
			newClause.getNegatedAtoms().addAll(first.getNegatedAtoms());
			newClause.getNegatedAtoms().addAll(second.getNegatedAtoms());
			newClause.getAtoms().remove(atom);
			newClause.getNegatedAtoms().remove(atom);
			clashedClauses.add(newClause);
		}
		return clashedClauses;
	}

	public static Set<String> getClashingAtoms(final Clause first, final Clause second) {
		Set<String> clashingAtoms = new HashSet<String>();
		for (String atom : first.getAtoms()) {
			for (String negAtom : second.getNegatedAtoms()) {
				if (atom.contentEquals(negAtom)) {
					clashingAtoms.add(atom);
				}
			}
		}
		for (String atom : second.getAtoms()) {
			for (String negAtom : first.getNegatedAtoms()) {
				if (atom.contentEquals(negAtom)) {
					clashingAtoms.add(atom);
				}
			}
		}
		return clashingAtoms;
	}

	public static boolean isClash(final Clause first, final Clause second) {
		return getClashingAtoms(first, second).isEmpty() == false;
	}
}
