package app.model;

import java.util.HashSet;
import java.util.Set;

import at.cl.atp.resolution.Atom;

public class Clause {

	public static Clause createAtomClause(final Atom atom) {
		Clause clause = new Clause();
		clause.atoms.add(atom.getName());
		return clause;
	}

	public static Clause createNegatedAtomClause(final Atom negatedAtom) {
		Clause clause = new Clause();
		clause.negatedAtoms.add(negatedAtom.getName());
		return clause;
	}

	private final Set<String> atoms;
	private final Set<String> negatedAtoms;
	private final Set<Clause> clashedWith;

	public Clause() {
		this.atoms = new HashSet<String>();
		this.negatedAtoms = new HashSet<String>();
		this.clashedWith = new HashSet<Clause>();
	}

	public Set<String> getAtoms() {
		return this.atoms;
	}

	public Set<String> getNegatedAtoms() {
		return this.negatedAtoms;
	}

	public boolean isEmpty() {
		return this.atoms.isEmpty() & this.negatedAtoms.isEmpty();
	}

	public boolean containsAtomNegated(final String atom) {
		return this.negatedAtoms.contains(atom);
	}

	public boolean containsAtomNonNegated(final String atom) {
		return this.atoms.contains(atom);
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		for (String atom : this.atoms) {
			builder.append(atom);
			builder.append(",");
		}
		for (String atom : this.negatedAtoms) {
			builder.append("!");
			builder.append(atom);
			builder.append(",");
		}
		builder.deleteCharAt(builder.length() - 1);
		return builder.toString();
	}

	public boolean isTautology() {
		int totalNumberOfAtoms = this.atoms.size() + this.negatedAtoms.size();
		Set<String> union = new HashSet<String>();
		union.addAll(this.atoms);
		union.addAll(this.negatedAtoms);
		if (union.size() < totalNumberOfAtoms) {
			return true;
		} else {
			return false;
		}
	}

	public boolean hasClashedWith(final Clause other) {
		if (this.clashedWith.contains(other)) {
			return true;
		} else {
			return false;
		}
	}

	public void registerClash(final Clause other) {
		this.clashedWith.add(other);
	}
}
