package util;

import at.cl.atp.resolution.AndFormula;
import at.cl.atp.resolution.Atom;
import at.cl.atp.resolution.Formula;
import at.cl.atp.resolution.ImplicationFormula;
import at.cl.atp.resolution.NegatedFormula;
import at.cl.atp.resolution.OrFormula;
import at.cl.atp.resolution.ResolutionProblem;

public class FormulaFormatter {

	public static String formulaToString(final ResolutionProblem problem) {
		StringBuilder builder = new StringBuilder();
		builder.append("{ ");
		int problemSize = problem.getFormulas().size();
		for (int i = 0; i < problemSize; i++) {
			Formula formula = problem.getFormulas().get(i);
			builder.append(formulaToString(formula));
			if ((i + 1) < problemSize) {
				builder.append(", ");
			}
		}
		builder.append(" }");
		return builder.toString();
	}

	public static String formulaToString(final Formula formula) {
		StringBuilder builder = new StringBuilder();
		formulaToString(formula, builder);
		return builder.toString();
	}

	private static void formulaToString(final Formula formula, final StringBuilder builder) {
		if (formula instanceof Atom) {
			builder.append(((Atom) formula).getName());
		} else if (formula instanceof NegatedFormula) {
			builder.append("!(");
			formulaToString(((NegatedFormula) formula).getInnerFormula(), builder);
			builder.append(")");
		} else if (formula instanceof AndFormula) {
			builder.append("and(");
			formulaToString(((AndFormula) formula).getFirst(), builder);
			builder.append(", ");
			formulaToString(((AndFormula) formula).getSecond(), builder);
			builder.append(")");
		} else if (formula instanceof OrFormula) {
			builder.append("or(");
			formulaToString(((OrFormula) formula).getFirst(), builder);
			builder.append(", ");
			formulaToString(((OrFormula) formula).getSecond(), builder);
			builder.append(")");
		} else if (formula instanceof ImplicationFormula) {
			builder.append("imp(");
			formulaToString(((ImplicationFormula) formula).getFirst(), builder);
			builder.append(", ");
			formulaToString(((ImplicationFormula) formula).getSecond(), builder);
			builder.append(")");
		} else {
			builder.append("NULL");
		}
	}
}
