package at.cl.atp.serializer;

import at.cl.atp.services.TPTPGrammarAccess;
import at.cl.atp.tPTP.CNF_annotated;
import at.cl.atp.tPTP.FOF_Constant;
import at.cl.atp.tPTP.FOF_Formula;
import at.cl.atp.tPTP.FOF_Function;
import at.cl.atp.tPTP.FOF_NegatedFormula;
import at.cl.atp.tPTP.FOF_Predicate;
import at.cl.atp.tPTP.FOF_QuantifiedFormula;
import at.cl.atp.tPTP.FOF_Variable;
import at.cl.atp.tPTP.TPTPPackage;
import at.cl.atp.tPTP.TPTP_File;
import com.google.inject.Inject;
import com.google.inject.Provider;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractDelegatingSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("all")
public class TPTPSemanticSequencer extends AbstractDelegatingSemanticSequencer {

	@Inject
	private TPTPGrammarAccess grammarAccess;
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == TPTPPackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case TPTPPackage.CNF_ANNOTATED:
				if(context == grammarAccess.getAnnotated_formulaRule() ||
				   context == grammarAccess.getCNF_annotatedRule()) {
					sequence_CNF_annotated(context, (CNF_annotated) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.FOF_CONSTANT:
				if(context == grammarAccess.getFOF_AtomRule() ||
				   context == grammarAccess.getFOF_ConstantRule() ||
				   context == grammarAccess.getFOF_PredicateArgumentRule()) {
					sequence_FOF_Constant(context, (FOF_Constant) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.FOF_FORMULA:
				if(context == grammarAccess.getFOF_BracedFormulaRule() ||
				   context == grammarAccess.getFOF_ExpressionRule() ||
				   context == grammarAccess.getFOF_ExpressionAccess().getFOF_FormulaLeftAction_1_0() ||
				   context == grammarAccess.getFOF_TermRule()) {
					sequence_FOF_Expression(context, (FOF_Formula) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.FOF_FUNCTION:
				if(context == grammarAccess.getFOF_FunctionRule() ||
				   context == grammarAccess.getFOF_PredicateArgumentRule()) {
					sequence_FOF_Function(context, (FOF_Function) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.FOF_NEGATED_FORMULA:
				if(context == grammarAccess.getFOF_BracedFormulaRule() ||
				   context == grammarAccess.getFOF_ExpressionRule() ||
				   context == grammarAccess.getFOF_ExpressionAccess().getFOF_FormulaLeftAction_1_0() ||
				   context == grammarAccess.getFOF_NegatedFormulaRule() ||
				   context == grammarAccess.getFOF_TermRule()) {
					sequence_FOF_NegatedFormula(context, (FOF_NegatedFormula) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.FOF_PREDICATE:
				if(context == grammarAccess.getFOF_BracedFormulaRule() ||
				   context == grammarAccess.getFOF_ExpressionRule() ||
				   context == grammarAccess.getFOF_ExpressionAccess().getFOF_FormulaLeftAction_1_0() ||
				   context == grammarAccess.getFOF_PredicateRule() ||
				   context == grammarAccess.getFOF_TermRule()) {
					sequence_FOF_Predicate(context, (FOF_Predicate) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.FOF_QUANTIFIED_FORMULA:
				if(context == grammarAccess.getFOF_BracedFormulaRule() ||
				   context == grammarAccess.getFOF_ExpressionRule() ||
				   context == grammarAccess.getFOF_ExpressionAccess().getFOF_FormulaLeftAction_1_0() ||
				   context == grammarAccess.getFOF_QuantifiedFormulaRule() ||
				   context == grammarAccess.getFOF_TermRule()) {
					sequence_FOF_QuantifiedFormula(context, (FOF_QuantifiedFormula) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.FOF_VARIABLE:
				if(context == grammarAccess.getFOF_AtomRule() ||
				   context == grammarAccess.getFOF_BracedFormulaRule() ||
				   context == grammarAccess.getFOF_ExpressionRule() ||
				   context == grammarAccess.getFOF_ExpressionAccess().getFOF_FormulaLeftAction_1_0() ||
				   context == grammarAccess.getFOF_PredicateArgumentRule() ||
				   context == grammarAccess.getFOF_TermRule() ||
				   context == grammarAccess.getFOF_VariableRule()) {
					sequence_FOF_Variable(context, (FOF_Variable) semanticObject); 
					return; 
				}
				else break;
			case TPTPPackage.TPTP_FILE:
				if(context == grammarAccess.getTPTP_FileRule()) {
					sequence_TPTP_File(context, (TPTP_File) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     (name=LOWERCASE_ID role=ROLE formula=FOF_Expression)
	 */
	protected void sequence_CNF_annotated(EObject context, CNF_annotated semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, TPTPPackage.Literals.CNF_ANNOTATED__NAME) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, TPTPPackage.Literals.CNF_ANNOTATED__NAME));
			if(transientValues.isValueTransient(semanticObject, TPTPPackage.Literals.CNF_ANNOTATED__ROLE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, TPTPPackage.Literals.CNF_ANNOTATED__ROLE));
			if(transientValues.isValueTransient(semanticObject, TPTPPackage.Literals.CNF_ANNOTATED__FORMULA) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, TPTPPackage.Literals.CNF_ANNOTATED__FORMULA));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getCNF_annotatedAccess().getNameLOWERCASE_IDTerminalRuleCall_2_0(), semanticObject.getName());
		feeder.accept(grammarAccess.getCNF_annotatedAccess().getRoleROLEEnumRuleCall_4_0(), semanticObject.getRole());
		feeder.accept(grammarAccess.getCNF_annotatedAccess().getFormulaFOF_ExpressionParserRuleCall_7_0(), semanticObject.getFormula());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     name=LOWERCASE_ID
	 */
	protected void sequence_FOF_Constant(EObject context, FOF_Constant semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (left=FOF_Expression_FOF_Formula_1_0 operation=BinaryOperation right=FOF_Expression)
	 */
	protected void sequence_FOF_Expression(EObject context, FOF_Formula semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, TPTPPackage.Literals.FOF_FORMULA__LEFT) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, TPTPPackage.Literals.FOF_FORMULA__LEFT));
			if(transientValues.isValueTransient(semanticObject, TPTPPackage.Literals.FOF_FORMULA__OPERATION) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, TPTPPackage.Literals.FOF_FORMULA__OPERATION));
			if(transientValues.isValueTransient(semanticObject, TPTPPackage.Literals.FOF_FORMULA__RIGHT) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, TPTPPackage.Literals.FOF_FORMULA__RIGHT));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getFOF_ExpressionAccess().getFOF_FormulaLeftAction_1_0(), semanticObject.getLeft());
		feeder.accept(grammarAccess.getFOF_ExpressionAccess().getOperationBinaryOperationEnumRuleCall_1_1_0(), semanticObject.getOperation());
		feeder.accept(grammarAccess.getFOF_ExpressionAccess().getRightFOF_ExpressionParserRuleCall_1_2_0(), semanticObject.getRight());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (name=LOWERCASE_ID (arguments+=FOF_PredicateArgument arguments+=FOF_PredicateArgument*)?)
	 */
	protected void sequence_FOF_Function(EObject context, FOF_Function semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     innerFormula=FOF_Term
	 */
	protected void sequence_FOF_NegatedFormula(EObject context, FOF_NegatedFormula semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (name=LOWERCASE_ID arguments+=FOF_PredicateArgument arguments+=FOF_PredicateArgument*)
	 */
	protected void sequence_FOF_Predicate(EObject context, FOF_Predicate semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (quantifier=Quantifier variables+=UPPERCASE_ID variables+=UPPERCASE_ID* innerFormula=FOF_Term)
	 */
	protected void sequence_FOF_QuantifiedFormula(EObject context, FOF_QuantifiedFormula semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     name=UPPERCASE_ID
	 */
	protected void sequence_FOF_Variable(EObject context, FOF_Variable semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     inputs+=Annotated_formula*
	 */
	protected void sequence_TPTP_File(EObject context, TPTP_File semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
}
